package owl.factories.jdd.bdd;

import com.google.common.base.MoreObjects;
import com.google.common.base.Preconditions;
import com.google.common.collect.Lists;
import com.google.common.primitives.Booleans;
import com.google.common.primitives.Floats;
import edu.umd.cs.findbugs.annotations.SuppressFBWarnings;
import java.util.ArrayList;
import javax.annotation.Generated;
import javax.annotation.Nullable;
import javax.annotation.ParametersAreNonnullByDefault;
import javax.annotation.concurrent.Immutable;
import javax.annotation.concurrent.NotThreadSafe;

/**
 * Immutable implementation of {@link BddConfiguration}.
 * <p>
 * Use the builder to create immutable instances:
 * {@code ImmutableBddConfiguration.builder()}.
 */
@SuppressWarnings({"PMD.AtLeastOneConstructor", "all"})
@SuppressFBWarnings
@ParametersAreNonnullByDefault
@Generated({"Immutables.generator", "BddConfiguration"})
@Immutable
public final class ImmutableBddConfiguration extends BddConfiguration {
  private final int cacheBinaryBinsPerHash;
  private final int cacheBinaryDivider;
  private final int cacheComposeDivider;
  private final int cacheSatisfactionBinsPerHash;
  private final int cacheSatisfactionDivider;
  private final int cacheTernaryBinsPerHash;
  private final int cacheTernaryDivider;
  private final int cacheUnaryBinsPerHash;
  private final int cacheUnaryDivider;
  private final int cacheVolatileBinsPerHash;
  private final int cacheVolatileMultiplier;
  private final int maximumNodeTableGrowth;
  private final int minimumCacheSize;
  private final int minimumDeadNodesCountForGcInGrow;
  private final int minimumFreeNodeCountAfterGc;
  private final float minimumFreeNodePercentageAfterGc;
  private final int minimumNodeTableGrowth;
  private final int minimumNodeTableSize;
  private final int nodeTableBigThreshold;
  private final int nodeTableSmallThreshold;
  private final boolean useGlobalComposeCache;
  private final boolean useShannonExists;

  private ImmutableBddConfiguration(ImmutableBddConfiguration.Builder builder) {
    if (builder.cacheBinaryBinsPerHashIsSet()) {
      initShim.cacheBinaryBinsPerHash(builder.cacheBinaryBinsPerHash);
    }
    if (builder.cacheBinaryDividerIsSet()) {
      initShim.cacheBinaryDivider(builder.cacheBinaryDivider);
    }
    if (builder.cacheComposeDividerIsSet()) {
      initShim.cacheComposeDivider(builder.cacheComposeDivider);
    }
    if (builder.cacheSatisfactionBinsPerHashIsSet()) {
      initShim.cacheSatisfactionBinsPerHash(builder.cacheSatisfactionBinsPerHash);
    }
    if (builder.cacheSatisfactionDividerIsSet()) {
      initShim.cacheSatisfactionDivider(builder.cacheSatisfactionDivider);
    }
    if (builder.cacheTernaryBinsPerHashIsSet()) {
      initShim.cacheTernaryBinsPerHash(builder.cacheTernaryBinsPerHash);
    }
    if (builder.cacheTernaryDividerIsSet()) {
      initShim.cacheTernaryDivider(builder.cacheTernaryDivider);
    }
    if (builder.cacheUnaryBinsPerHashIsSet()) {
      initShim.cacheUnaryBinsPerHash(builder.cacheUnaryBinsPerHash);
    }
    if (builder.cacheUnaryDividerIsSet()) {
      initShim.cacheUnaryDivider(builder.cacheUnaryDivider);
    }
    if (builder.cacheVolatileBinsPerHashIsSet()) {
      initShim.cacheVolatileBinsPerHash(builder.cacheVolatileBinsPerHash);
    }
    if (builder.cacheVolatileMultiplierIsSet()) {
      initShim.cacheVolatileMultiplier(builder.cacheVolatileMultiplier);
    }
    if (builder.maximumNodeTableGrowthIsSet()) {
      initShim.maximumNodeTableGrowth(builder.maximumNodeTableGrowth);
    }
    if (builder.minimumCacheSizeIsSet()) {
      initShim.minimumCacheSize(builder.minimumCacheSize);
    }
    if (builder.minimumDeadNodesCountForGcInGrowIsSet()) {
      initShim.minimumDeadNodesCountForGcInGrow(builder.minimumDeadNodesCountForGcInGrow);
    }
    if (builder.minimumFreeNodeCountAfterGcIsSet()) {
      initShim.minimumFreeNodeCountAfterGc(builder.minimumFreeNodeCountAfterGc);
    }
    if (builder.minimumFreeNodePercentageAfterGcIsSet()) {
      initShim.minimumFreeNodePercentageAfterGc(builder.minimumFreeNodePercentageAfterGc);
    }
    if (builder.minimumNodeTableGrowthIsSet()) {
      initShim.minimumNodeTableGrowth(builder.minimumNodeTableGrowth);
    }
    if (builder.minimumNodeTableSizeIsSet()) {
      initShim.minimumNodeTableSize(builder.minimumNodeTableSize);
    }
    if (builder.nodeTableBigThresholdIsSet()) {
      initShim.nodeTableBigThreshold(builder.nodeTableBigThreshold);
    }
    if (builder.nodeTableSmallThresholdIsSet()) {
      initShim.nodeTableSmallThreshold(builder.nodeTableSmallThreshold);
    }
    if (builder.useGlobalComposeCacheIsSet()) {
      initShim.useGlobalComposeCache(builder.useGlobalComposeCache);
    }
    if (builder.useShannonExistsIsSet()) {
      initShim.useShannonExists(builder.useShannonExists);
    }
    this.cacheBinaryBinsPerHash = initShim.cacheBinaryBinsPerHash();
    this.cacheBinaryDivider = initShim.cacheBinaryDivider();
    this.cacheComposeDivider = initShim.cacheComposeDivider();
    this.cacheSatisfactionBinsPerHash = initShim.cacheSatisfactionBinsPerHash();
    this.cacheSatisfactionDivider = initShim.cacheSatisfactionDivider();
    this.cacheTernaryBinsPerHash = initShim.cacheTernaryBinsPerHash();
    this.cacheTernaryDivider = initShim.cacheTernaryDivider();
    this.cacheUnaryBinsPerHash = initShim.cacheUnaryBinsPerHash();
    this.cacheUnaryDivider = initShim.cacheUnaryDivider();
    this.cacheVolatileBinsPerHash = initShim.cacheVolatileBinsPerHash();
    this.cacheVolatileMultiplier = initShim.cacheVolatileMultiplier();
    this.maximumNodeTableGrowth = initShim.maximumNodeTableGrowth();
    this.minimumCacheSize = initShim.minimumCacheSize();
    this.minimumDeadNodesCountForGcInGrow = initShim.minimumDeadNodesCountForGcInGrow();
    this.minimumFreeNodeCountAfterGc = initShim.minimumFreeNodeCountAfterGc();
    this.minimumFreeNodePercentageAfterGc = initShim.minimumFreeNodePercentageAfterGc();
    this.minimumNodeTableGrowth = initShim.minimumNodeTableGrowth();
    this.minimumNodeTableSize = initShim.minimumNodeTableSize();
    this.nodeTableBigThreshold = initShim.nodeTableBigThreshold();
    this.nodeTableSmallThreshold = initShim.nodeTableSmallThreshold();
    this.useGlobalComposeCache = initShim.useGlobalComposeCache();
    this.useShannonExists = initShim.useShannonExists();
    this.initShim = null;
  }

  private ImmutableBddConfiguration(
      int cacheBinaryBinsPerHash,
      int cacheBinaryDivider,
      int cacheComposeDivider,
      int cacheSatisfactionBinsPerHash,
      int cacheSatisfactionDivider,
      int cacheTernaryBinsPerHash,
      int cacheTernaryDivider,
      int cacheUnaryBinsPerHash,
      int cacheUnaryDivider,
      int cacheVolatileBinsPerHash,
      int cacheVolatileMultiplier,
      int maximumNodeTableGrowth,
      int minimumCacheSize,
      int minimumDeadNodesCountForGcInGrow,
      int minimumFreeNodeCountAfterGc,
      float minimumFreeNodePercentageAfterGc,
      int minimumNodeTableGrowth,
      int minimumNodeTableSize,
      int nodeTableBigThreshold,
      int nodeTableSmallThreshold,
      boolean useGlobalComposeCache,
      boolean useShannonExists) {
    this.cacheBinaryBinsPerHash = cacheBinaryBinsPerHash;
    this.cacheBinaryDivider = cacheBinaryDivider;
    this.cacheComposeDivider = cacheComposeDivider;
    this.cacheSatisfactionBinsPerHash = cacheSatisfactionBinsPerHash;
    this.cacheSatisfactionDivider = cacheSatisfactionDivider;
    this.cacheTernaryBinsPerHash = cacheTernaryBinsPerHash;
    this.cacheTernaryDivider = cacheTernaryDivider;
    this.cacheUnaryBinsPerHash = cacheUnaryBinsPerHash;
    this.cacheUnaryDivider = cacheUnaryDivider;
    this.cacheVolatileBinsPerHash = cacheVolatileBinsPerHash;
    this.cacheVolatileMultiplier = cacheVolatileMultiplier;
    this.maximumNodeTableGrowth = maximumNodeTableGrowth;
    this.minimumCacheSize = minimumCacheSize;
    this.minimumDeadNodesCountForGcInGrow = minimumDeadNodesCountForGcInGrow;
    this.minimumFreeNodeCountAfterGc = minimumFreeNodeCountAfterGc;
    this.minimumFreeNodePercentageAfterGc = minimumFreeNodePercentageAfterGc;
    this.minimumNodeTableGrowth = minimumNodeTableGrowth;
    this.minimumNodeTableSize = minimumNodeTableSize;
    this.nodeTableBigThreshold = nodeTableBigThreshold;
    this.nodeTableSmallThreshold = nodeTableSmallThreshold;
    this.useGlobalComposeCache = useGlobalComposeCache;
    this.useShannonExists = useShannonExists;
    this.initShim = null;
  }

  private static final int STAGE_INITIALIZING = -1;
  private static final int STAGE_UNINITIALIZED = 0;
  private static final int STAGE_INITIALIZED = 1;
  private transient volatile InitShim initShim = new InitShim();

  private final class InitShim {
    private int cacheBinaryBinsPerHash;
    private int cacheBinaryBinsPerHashBuildStage;

    int cacheBinaryBinsPerHash() {
      if (cacheBinaryBinsPerHashBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (cacheBinaryBinsPerHashBuildStage == STAGE_UNINITIALIZED) {
        cacheBinaryBinsPerHashBuildStage = STAGE_INITIALIZING;
        this.cacheBinaryBinsPerHash = ImmutableBddConfiguration.super.cacheBinaryBinsPerHash();
        cacheBinaryBinsPerHashBuildStage = STAGE_INITIALIZED;
      }
      return this.cacheBinaryBinsPerHash;
    }

    void cacheBinaryBinsPerHash(int cacheBinaryBinsPerHash) {
      this.cacheBinaryBinsPerHash = cacheBinaryBinsPerHash;
      cacheBinaryBinsPerHashBuildStage = STAGE_INITIALIZED;
    }
    private int cacheBinaryDivider;
    private int cacheBinaryDividerBuildStage;

    int cacheBinaryDivider() {
      if (cacheBinaryDividerBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (cacheBinaryDividerBuildStage == STAGE_UNINITIALIZED) {
        cacheBinaryDividerBuildStage = STAGE_INITIALIZING;
        this.cacheBinaryDivider = ImmutableBddConfiguration.super.cacheBinaryDivider();
        cacheBinaryDividerBuildStage = STAGE_INITIALIZED;
      }
      return this.cacheBinaryDivider;
    }

    void cacheBinaryDivider(int cacheBinaryDivider) {
      this.cacheBinaryDivider = cacheBinaryDivider;
      cacheBinaryDividerBuildStage = STAGE_INITIALIZED;
    }
    private int cacheComposeDivider;
    private int cacheComposeDividerBuildStage;

    int cacheComposeDivider() {
      if (cacheComposeDividerBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (cacheComposeDividerBuildStage == STAGE_UNINITIALIZED) {
        cacheComposeDividerBuildStage = STAGE_INITIALIZING;
        this.cacheComposeDivider = ImmutableBddConfiguration.super.cacheComposeDivider();
        cacheComposeDividerBuildStage = STAGE_INITIALIZED;
      }
      return this.cacheComposeDivider;
    }

    void cacheComposeDivider(int cacheComposeDivider) {
      this.cacheComposeDivider = cacheComposeDivider;
      cacheComposeDividerBuildStage = STAGE_INITIALIZED;
    }
    private int cacheSatisfactionBinsPerHash;
    private int cacheSatisfactionBinsPerHashBuildStage;

    int cacheSatisfactionBinsPerHash() {
      if (cacheSatisfactionBinsPerHashBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (cacheSatisfactionBinsPerHashBuildStage == STAGE_UNINITIALIZED) {
        cacheSatisfactionBinsPerHashBuildStage = STAGE_INITIALIZING;
        this.cacheSatisfactionBinsPerHash = ImmutableBddConfiguration.super.cacheSatisfactionBinsPerHash();
        cacheSatisfactionBinsPerHashBuildStage = STAGE_INITIALIZED;
      }
      return this.cacheSatisfactionBinsPerHash;
    }

    void cacheSatisfactionBinsPerHash(int cacheSatisfactionBinsPerHash) {
      this.cacheSatisfactionBinsPerHash = cacheSatisfactionBinsPerHash;
      cacheSatisfactionBinsPerHashBuildStage = STAGE_INITIALIZED;
    }
    private int cacheSatisfactionDivider;
    private int cacheSatisfactionDividerBuildStage;

    int cacheSatisfactionDivider() {
      if (cacheSatisfactionDividerBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (cacheSatisfactionDividerBuildStage == STAGE_UNINITIALIZED) {
        cacheSatisfactionDividerBuildStage = STAGE_INITIALIZING;
        this.cacheSatisfactionDivider = ImmutableBddConfiguration.super.cacheSatisfactionDivider();
        cacheSatisfactionDividerBuildStage = STAGE_INITIALIZED;
      }
      return this.cacheSatisfactionDivider;
    }

    void cacheSatisfactionDivider(int cacheSatisfactionDivider) {
      this.cacheSatisfactionDivider = cacheSatisfactionDivider;
      cacheSatisfactionDividerBuildStage = STAGE_INITIALIZED;
    }
    private int cacheTernaryBinsPerHash;
    private int cacheTernaryBinsPerHashBuildStage;

    int cacheTernaryBinsPerHash() {
      if (cacheTernaryBinsPerHashBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (cacheTernaryBinsPerHashBuildStage == STAGE_UNINITIALIZED) {
        cacheTernaryBinsPerHashBuildStage = STAGE_INITIALIZING;
        this.cacheTernaryBinsPerHash = ImmutableBddConfiguration.super.cacheTernaryBinsPerHash();
        cacheTernaryBinsPerHashBuildStage = STAGE_INITIALIZED;
      }
      return this.cacheTernaryBinsPerHash;
    }

    void cacheTernaryBinsPerHash(int cacheTernaryBinsPerHash) {
      this.cacheTernaryBinsPerHash = cacheTernaryBinsPerHash;
      cacheTernaryBinsPerHashBuildStage = STAGE_INITIALIZED;
    }
    private int cacheTernaryDivider;
    private int cacheTernaryDividerBuildStage;

    int cacheTernaryDivider() {
      if (cacheTernaryDividerBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (cacheTernaryDividerBuildStage == STAGE_UNINITIALIZED) {
        cacheTernaryDividerBuildStage = STAGE_INITIALIZING;
        this.cacheTernaryDivider = ImmutableBddConfiguration.super.cacheTernaryDivider();
        cacheTernaryDividerBuildStage = STAGE_INITIALIZED;
      }
      return this.cacheTernaryDivider;
    }

    void cacheTernaryDivider(int cacheTernaryDivider) {
      this.cacheTernaryDivider = cacheTernaryDivider;
      cacheTernaryDividerBuildStage = STAGE_INITIALIZED;
    }
    private int cacheUnaryBinsPerHash;
    private int cacheUnaryBinsPerHashBuildStage;

    int cacheUnaryBinsPerHash() {
      if (cacheUnaryBinsPerHashBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (cacheUnaryBinsPerHashBuildStage == STAGE_UNINITIALIZED) {
        cacheUnaryBinsPerHashBuildStage = STAGE_INITIALIZING;
        this.cacheUnaryBinsPerHash = ImmutableBddConfiguration.super.cacheUnaryBinsPerHash();
        cacheUnaryBinsPerHashBuildStage = STAGE_INITIALIZED;
      }
      return this.cacheUnaryBinsPerHash;
    }

    void cacheUnaryBinsPerHash(int cacheUnaryBinsPerHash) {
      this.cacheUnaryBinsPerHash = cacheUnaryBinsPerHash;
      cacheUnaryBinsPerHashBuildStage = STAGE_INITIALIZED;
    }
    private int cacheUnaryDivider;
    private int cacheUnaryDividerBuildStage;

    int cacheUnaryDivider() {
      if (cacheUnaryDividerBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (cacheUnaryDividerBuildStage == STAGE_UNINITIALIZED) {
        cacheUnaryDividerBuildStage = STAGE_INITIALIZING;
        this.cacheUnaryDivider = ImmutableBddConfiguration.super.cacheUnaryDivider();
        cacheUnaryDividerBuildStage = STAGE_INITIALIZED;
      }
      return this.cacheUnaryDivider;
    }

    void cacheUnaryDivider(int cacheUnaryDivider) {
      this.cacheUnaryDivider = cacheUnaryDivider;
      cacheUnaryDividerBuildStage = STAGE_INITIALIZED;
    }
    private int cacheVolatileBinsPerHash;
    private int cacheVolatileBinsPerHashBuildStage;

    int cacheVolatileBinsPerHash() {
      if (cacheVolatileBinsPerHashBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (cacheVolatileBinsPerHashBuildStage == STAGE_UNINITIALIZED) {
        cacheVolatileBinsPerHashBuildStage = STAGE_INITIALIZING;
        this.cacheVolatileBinsPerHash = ImmutableBddConfiguration.super.cacheVolatileBinsPerHash();
        cacheVolatileBinsPerHashBuildStage = STAGE_INITIALIZED;
      }
      return this.cacheVolatileBinsPerHash;
    }

    void cacheVolatileBinsPerHash(int cacheVolatileBinsPerHash) {
      this.cacheVolatileBinsPerHash = cacheVolatileBinsPerHash;
      cacheVolatileBinsPerHashBuildStage = STAGE_INITIALIZED;
    }
    private int cacheVolatileMultiplier;
    private int cacheVolatileMultiplierBuildStage;

    int cacheVolatileMultiplier() {
      if (cacheVolatileMultiplierBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (cacheVolatileMultiplierBuildStage == STAGE_UNINITIALIZED) {
        cacheVolatileMultiplierBuildStage = STAGE_INITIALIZING;
        this.cacheVolatileMultiplier = ImmutableBddConfiguration.super.cacheVolatileMultiplier();
        cacheVolatileMultiplierBuildStage = STAGE_INITIALIZED;
      }
      return this.cacheVolatileMultiplier;
    }

    void cacheVolatileMultiplier(int cacheVolatileMultiplier) {
      this.cacheVolatileMultiplier = cacheVolatileMultiplier;
      cacheVolatileMultiplierBuildStage = STAGE_INITIALIZED;
    }
    private int maximumNodeTableGrowth;
    private int maximumNodeTableGrowthBuildStage;

    int maximumNodeTableGrowth() {
      if (maximumNodeTableGrowthBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (maximumNodeTableGrowthBuildStage == STAGE_UNINITIALIZED) {
        maximumNodeTableGrowthBuildStage = STAGE_INITIALIZING;
        this.maximumNodeTableGrowth = ImmutableBddConfiguration.super.maximumNodeTableGrowth();
        maximumNodeTableGrowthBuildStage = STAGE_INITIALIZED;
      }
      return this.maximumNodeTableGrowth;
    }

    void maximumNodeTableGrowth(int maximumNodeTableGrowth) {
      this.maximumNodeTableGrowth = maximumNodeTableGrowth;
      maximumNodeTableGrowthBuildStage = STAGE_INITIALIZED;
    }
    private int minimumCacheSize;
    private int minimumCacheSizeBuildStage;

    int minimumCacheSize() {
      if (minimumCacheSizeBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (minimumCacheSizeBuildStage == STAGE_UNINITIALIZED) {
        minimumCacheSizeBuildStage = STAGE_INITIALIZING;
        this.minimumCacheSize = ImmutableBddConfiguration.super.minimumCacheSize();
        minimumCacheSizeBuildStage = STAGE_INITIALIZED;
      }
      return this.minimumCacheSize;
    }

    void minimumCacheSize(int minimumCacheSize) {
      this.minimumCacheSize = minimumCacheSize;
      minimumCacheSizeBuildStage = STAGE_INITIALIZED;
    }
    private int minimumDeadNodesCountForGcInGrow;
    private int minimumDeadNodesCountForGcInGrowBuildStage;

    int minimumDeadNodesCountForGcInGrow() {
      if (minimumDeadNodesCountForGcInGrowBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (minimumDeadNodesCountForGcInGrowBuildStage == STAGE_UNINITIALIZED) {
        minimumDeadNodesCountForGcInGrowBuildStage = STAGE_INITIALIZING;
        this.minimumDeadNodesCountForGcInGrow = ImmutableBddConfiguration.super.minimumDeadNodesCountForGcInGrow();
        minimumDeadNodesCountForGcInGrowBuildStage = STAGE_INITIALIZED;
      }
      return this.minimumDeadNodesCountForGcInGrow;
    }

    void minimumDeadNodesCountForGcInGrow(int minimumDeadNodesCountForGcInGrow) {
      this.minimumDeadNodesCountForGcInGrow = minimumDeadNodesCountForGcInGrow;
      minimumDeadNodesCountForGcInGrowBuildStage = STAGE_INITIALIZED;
    }
    private int minimumFreeNodeCountAfterGc;
    private int minimumFreeNodeCountAfterGcBuildStage;

    int minimumFreeNodeCountAfterGc() {
      if (minimumFreeNodeCountAfterGcBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (minimumFreeNodeCountAfterGcBuildStage == STAGE_UNINITIALIZED) {
        minimumFreeNodeCountAfterGcBuildStage = STAGE_INITIALIZING;
        this.minimumFreeNodeCountAfterGc = ImmutableBddConfiguration.super.minimumFreeNodeCountAfterGc();
        minimumFreeNodeCountAfterGcBuildStage = STAGE_INITIALIZED;
      }
      return this.minimumFreeNodeCountAfterGc;
    }

    void minimumFreeNodeCountAfterGc(int minimumFreeNodeCountAfterGc) {
      this.minimumFreeNodeCountAfterGc = minimumFreeNodeCountAfterGc;
      minimumFreeNodeCountAfterGcBuildStage = STAGE_INITIALIZED;
    }
    private float minimumFreeNodePercentageAfterGc;
    private int minimumFreeNodePercentageAfterGcBuildStage;

    float minimumFreeNodePercentageAfterGc() {
      if (minimumFreeNodePercentageAfterGcBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (minimumFreeNodePercentageAfterGcBuildStage == STAGE_UNINITIALIZED) {
        minimumFreeNodePercentageAfterGcBuildStage = STAGE_INITIALIZING;
        this.minimumFreeNodePercentageAfterGc = ImmutableBddConfiguration.super.minimumFreeNodePercentageAfterGc();
        minimumFreeNodePercentageAfterGcBuildStage = STAGE_INITIALIZED;
      }
      return this.minimumFreeNodePercentageAfterGc;
    }

    void minimumFreeNodePercentageAfterGc(float minimumFreeNodePercentageAfterGc) {
      this.minimumFreeNodePercentageAfterGc = minimumFreeNodePercentageAfterGc;
      minimumFreeNodePercentageAfterGcBuildStage = STAGE_INITIALIZED;
    }
    private int minimumNodeTableGrowth;
    private int minimumNodeTableGrowthBuildStage;

    int minimumNodeTableGrowth() {
      if (minimumNodeTableGrowthBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (minimumNodeTableGrowthBuildStage == STAGE_UNINITIALIZED) {
        minimumNodeTableGrowthBuildStage = STAGE_INITIALIZING;
        this.minimumNodeTableGrowth = ImmutableBddConfiguration.super.minimumNodeTableGrowth();
        minimumNodeTableGrowthBuildStage = STAGE_INITIALIZED;
      }
      return this.minimumNodeTableGrowth;
    }

    void minimumNodeTableGrowth(int minimumNodeTableGrowth) {
      this.minimumNodeTableGrowth = minimumNodeTableGrowth;
      minimumNodeTableGrowthBuildStage = STAGE_INITIALIZED;
    }
    private int minimumNodeTableSize;
    private int minimumNodeTableSizeBuildStage;

    int minimumNodeTableSize() {
      if (minimumNodeTableSizeBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (minimumNodeTableSizeBuildStage == STAGE_UNINITIALIZED) {
        minimumNodeTableSizeBuildStage = STAGE_INITIALIZING;
        this.minimumNodeTableSize = ImmutableBddConfiguration.super.minimumNodeTableSize();
        minimumNodeTableSizeBuildStage = STAGE_INITIALIZED;
      }
      return this.minimumNodeTableSize;
    }

    void minimumNodeTableSize(int minimumNodeTableSize) {
      this.minimumNodeTableSize = minimumNodeTableSize;
      minimumNodeTableSizeBuildStage = STAGE_INITIALIZED;
    }
    private int nodeTableBigThreshold;
    private int nodeTableBigThresholdBuildStage;

    int nodeTableBigThreshold() {
      if (nodeTableBigThresholdBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (nodeTableBigThresholdBuildStage == STAGE_UNINITIALIZED) {
        nodeTableBigThresholdBuildStage = STAGE_INITIALIZING;
        this.nodeTableBigThreshold = ImmutableBddConfiguration.super.nodeTableBigThreshold();
        nodeTableBigThresholdBuildStage = STAGE_INITIALIZED;
      }
      return this.nodeTableBigThreshold;
    }

    void nodeTableBigThreshold(int nodeTableBigThreshold) {
      this.nodeTableBigThreshold = nodeTableBigThreshold;
      nodeTableBigThresholdBuildStage = STAGE_INITIALIZED;
    }
    private int nodeTableSmallThreshold;
    private int nodeTableSmallThresholdBuildStage;

    int nodeTableSmallThreshold() {
      if (nodeTableSmallThresholdBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (nodeTableSmallThresholdBuildStage == STAGE_UNINITIALIZED) {
        nodeTableSmallThresholdBuildStage = STAGE_INITIALIZING;
        this.nodeTableSmallThreshold = ImmutableBddConfiguration.super.nodeTableSmallThreshold();
        nodeTableSmallThresholdBuildStage = STAGE_INITIALIZED;
      }
      return this.nodeTableSmallThreshold;
    }

    void nodeTableSmallThreshold(int nodeTableSmallThreshold) {
      this.nodeTableSmallThreshold = nodeTableSmallThreshold;
      nodeTableSmallThresholdBuildStage = STAGE_INITIALIZED;
    }
    private boolean useGlobalComposeCache;
    private int useGlobalComposeCacheBuildStage;

    boolean useGlobalComposeCache() {
      if (useGlobalComposeCacheBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (useGlobalComposeCacheBuildStage == STAGE_UNINITIALIZED) {
        useGlobalComposeCacheBuildStage = STAGE_INITIALIZING;
        this.useGlobalComposeCache = ImmutableBddConfiguration.super.useGlobalComposeCache();
        useGlobalComposeCacheBuildStage = STAGE_INITIALIZED;
      }
      return this.useGlobalComposeCache;
    }

    void useGlobalComposeCache(boolean useGlobalComposeCache) {
      this.useGlobalComposeCache = useGlobalComposeCache;
      useGlobalComposeCacheBuildStage = STAGE_INITIALIZED;
    }
    private boolean useShannonExists;
    private int useShannonExistsBuildStage;

    boolean useShannonExists() {
      if (useShannonExistsBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (useShannonExistsBuildStage == STAGE_UNINITIALIZED) {
        useShannonExistsBuildStage = STAGE_INITIALIZING;
        this.useShannonExists = ImmutableBddConfiguration.super.useShannonExists();
        useShannonExistsBuildStage = STAGE_INITIALIZED;
      }
      return this.useShannonExists;
    }

    void useShannonExists(boolean useShannonExists) {
      this.useShannonExists = useShannonExists;
      useShannonExistsBuildStage = STAGE_INITIALIZED;
    }

    private String formatInitCycleMessage() {
      ArrayList<String> attributes = Lists.newArrayList();
      if (cacheBinaryBinsPerHashBuildStage == STAGE_INITIALIZING) attributes.add("cacheBinaryBinsPerHash");
      if (cacheBinaryDividerBuildStage == STAGE_INITIALIZING) attributes.add("cacheBinaryDivider");
      if (cacheComposeDividerBuildStage == STAGE_INITIALIZING) attributes.add("cacheComposeDivider");
      if (cacheSatisfactionBinsPerHashBuildStage == STAGE_INITIALIZING) attributes.add("cacheSatisfactionBinsPerHash");
      if (cacheSatisfactionDividerBuildStage == STAGE_INITIALIZING) attributes.add("cacheSatisfactionDivider");
      if (cacheTernaryBinsPerHashBuildStage == STAGE_INITIALIZING) attributes.add("cacheTernaryBinsPerHash");
      if (cacheTernaryDividerBuildStage == STAGE_INITIALIZING) attributes.add("cacheTernaryDivider");
      if (cacheUnaryBinsPerHashBuildStage == STAGE_INITIALIZING) attributes.add("cacheUnaryBinsPerHash");
      if (cacheUnaryDividerBuildStage == STAGE_INITIALIZING) attributes.add("cacheUnaryDivider");
      if (cacheVolatileBinsPerHashBuildStage == STAGE_INITIALIZING) attributes.add("cacheVolatileBinsPerHash");
      if (cacheVolatileMultiplierBuildStage == STAGE_INITIALIZING) attributes.add("cacheVolatileMultiplier");
      if (maximumNodeTableGrowthBuildStage == STAGE_INITIALIZING) attributes.add("maximumNodeTableGrowth");
      if (minimumCacheSizeBuildStage == STAGE_INITIALIZING) attributes.add("minimumCacheSize");
      if (minimumDeadNodesCountForGcInGrowBuildStage == STAGE_INITIALIZING) attributes.add("minimumDeadNodesCountForGcInGrow");
      if (minimumFreeNodeCountAfterGcBuildStage == STAGE_INITIALIZING) attributes.add("minimumFreeNodeCountAfterGc");
      if (minimumFreeNodePercentageAfterGcBuildStage == STAGE_INITIALIZING) attributes.add("minimumFreeNodePercentageAfterGc");
      if (minimumNodeTableGrowthBuildStage == STAGE_INITIALIZING) attributes.add("minimumNodeTableGrowth");
      if (minimumNodeTableSizeBuildStage == STAGE_INITIALIZING) attributes.add("minimumNodeTableSize");
      if (nodeTableBigThresholdBuildStage == STAGE_INITIALIZING) attributes.add("nodeTableBigThreshold");
      if (nodeTableSmallThresholdBuildStage == STAGE_INITIALIZING) attributes.add("nodeTableSmallThreshold");
      if (useGlobalComposeCacheBuildStage == STAGE_INITIALIZING) attributes.add("useGlobalComposeCache");
      if (useShannonExistsBuildStage == STAGE_INITIALIZING) attributes.add("useShannonExists");
      return "Cannot build BddConfiguration, attribute initializers form cycle" + attributes;
    }
  }

  /**
   * @return The value of the {@code cacheBinaryBinsPerHash} attribute
   */
  @Override
  public int cacheBinaryBinsPerHash() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.cacheBinaryBinsPerHash()
        : this.cacheBinaryBinsPerHash;
  }

  /**
   * @return The value of the {@code cacheBinaryDivider} attribute
   */
  @Override
  public int cacheBinaryDivider() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.cacheBinaryDivider()
        : this.cacheBinaryDivider;
  }

  /**
   * @return The value of the {@code cacheComposeDivider} attribute
   */
  @Override
  public int cacheComposeDivider() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.cacheComposeDivider()
        : this.cacheComposeDivider;
  }

  /**
   * @return The value of the {@code cacheSatisfactionBinsPerHash} attribute
   */
  @Override
  public int cacheSatisfactionBinsPerHash() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.cacheSatisfactionBinsPerHash()
        : this.cacheSatisfactionBinsPerHash;
  }

  /**
   * @return The value of the {@code cacheSatisfactionDivider} attribute
   */
  @Override
  public int cacheSatisfactionDivider() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.cacheSatisfactionDivider()
        : this.cacheSatisfactionDivider;
  }

  /**
   * @return The value of the {@code cacheTernaryBinsPerHash} attribute
   */
  @Override
  public int cacheTernaryBinsPerHash() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.cacheTernaryBinsPerHash()
        : this.cacheTernaryBinsPerHash;
  }

  /**
   * @return The value of the {@code cacheTernaryDivider} attribute
   */
  @Override
  public int cacheTernaryDivider() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.cacheTernaryDivider()
        : this.cacheTernaryDivider;
  }

  /**
   * @return The value of the {@code cacheUnaryBinsPerHash} attribute
   */
  @Override
  public int cacheUnaryBinsPerHash() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.cacheUnaryBinsPerHash()
        : this.cacheUnaryBinsPerHash;
  }

  /**
   * @return The value of the {@code cacheUnaryDivider} attribute
   */
  @Override
  public int cacheUnaryDivider() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.cacheUnaryDivider()
        : this.cacheUnaryDivider;
  }

  /**
   * @return The value of the {@code cacheVolatileBinsPerHash} attribute
   */
  @Override
  public int cacheVolatileBinsPerHash() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.cacheVolatileBinsPerHash()
        : this.cacheVolatileBinsPerHash;
  }

  /**
   * @return The value of the {@code cacheVolatileMultiplier} attribute
   */
  @Override
  public int cacheVolatileMultiplier() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.cacheVolatileMultiplier()
        : this.cacheVolatileMultiplier;
  }

  /**
   * @return The value of the {@code maximumNodeTableGrowth} attribute
   */
  @Override
  public int maximumNodeTableGrowth() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.maximumNodeTableGrowth()
        : this.maximumNodeTableGrowth;
  }

  /**
   * @return The value of the {@code minimumCacheSize} attribute
   */
  @Override
  public int minimumCacheSize() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.minimumCacheSize()
        : this.minimumCacheSize;
  }

  /**
   * @return The value of the {@code minimumDeadNodesCountForGcInGrow} attribute
   */
  @Override
  public int minimumDeadNodesCountForGcInGrow() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.minimumDeadNodesCountForGcInGrow()
        : this.minimumDeadNodesCountForGcInGrow;
  }

  /**
   * @return The value of the {@code minimumFreeNodeCountAfterGc} attribute
   */
  @Override
  public int minimumFreeNodeCountAfterGc() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.minimumFreeNodeCountAfterGc()
        : this.minimumFreeNodeCountAfterGc;
  }

  /**
   * @return The value of the {@code minimumFreeNodePercentageAfterGc} attribute
   */
  @Override
  public float minimumFreeNodePercentageAfterGc() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.minimumFreeNodePercentageAfterGc()
        : this.minimumFreeNodePercentageAfterGc;
  }

  /**
   * @return The value of the {@code minimumNodeTableGrowth} attribute
   */
  @Override
  public int minimumNodeTableGrowth() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.minimumNodeTableGrowth()
        : this.minimumNodeTableGrowth;
  }

  /**
   * @return The value of the {@code minimumNodeTableSize} attribute
   */
  @Override
  public int minimumNodeTableSize() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.minimumNodeTableSize()
        : this.minimumNodeTableSize;
  }

  /**
   * @return The value of the {@code nodeTableBigThreshold} attribute
   */
  @Override
  public int nodeTableBigThreshold() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.nodeTableBigThreshold()
        : this.nodeTableBigThreshold;
  }

  /**
   * @return The value of the {@code nodeTableSmallThreshold} attribute
   */
  @Override
  public int nodeTableSmallThreshold() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.nodeTableSmallThreshold()
        : this.nodeTableSmallThreshold;
  }

  /**
   * @return The value of the {@code useGlobalComposeCache} attribute
   */
  @Override
  public boolean useGlobalComposeCache() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.useGlobalComposeCache()
        : this.useGlobalComposeCache;
  }

  /**
   * @return The value of the {@code useShannonExists} attribute
   */
  @Override
  public boolean useShannonExists() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.useShannonExists()
        : this.useShannonExists;
  }

  /**
   * Copy the current immutable object by setting a value for the {@link BddConfiguration#cacheBinaryBinsPerHash() cacheBinaryBinsPerHash} attribute.
   * A value equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for cacheBinaryBinsPerHash
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableBddConfiguration withCacheBinaryBinsPerHash(int value) {
    if (this.cacheBinaryBinsPerHash == value) return this;
    return new ImmutableBddConfiguration(
        value,
        this.cacheBinaryDivider,
        this.cacheComposeDivider,
        this.cacheSatisfactionBinsPerHash,
        this.cacheSatisfactionDivider,
        this.cacheTernaryBinsPerHash,
        this.cacheTernaryDivider,
        this.cacheUnaryBinsPerHash,
        this.cacheUnaryDivider,
        this.cacheVolatileBinsPerHash,
        this.cacheVolatileMultiplier,
        this.maximumNodeTableGrowth,
        this.minimumCacheSize,
        this.minimumDeadNodesCountForGcInGrow,
        this.minimumFreeNodeCountAfterGc,
        this.minimumFreeNodePercentageAfterGc,
        this.minimumNodeTableGrowth,
        this.minimumNodeTableSize,
        this.nodeTableBigThreshold,
        this.nodeTableSmallThreshold,
        this.useGlobalComposeCache,
        this.useShannonExists);
  }

  /**
   * Copy the current immutable object by setting a value for the {@link BddConfiguration#cacheBinaryDivider() cacheBinaryDivider} attribute.
   * A value equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for cacheBinaryDivider
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableBddConfiguration withCacheBinaryDivider(int value) {
    if (this.cacheBinaryDivider == value) return this;
    return new ImmutableBddConfiguration(
        this.cacheBinaryBinsPerHash,
        value,
        this.cacheComposeDivider,
        this.cacheSatisfactionBinsPerHash,
        this.cacheSatisfactionDivider,
        this.cacheTernaryBinsPerHash,
        this.cacheTernaryDivider,
        this.cacheUnaryBinsPerHash,
        this.cacheUnaryDivider,
        this.cacheVolatileBinsPerHash,
        this.cacheVolatileMultiplier,
        this.maximumNodeTableGrowth,
        this.minimumCacheSize,
        this.minimumDeadNodesCountForGcInGrow,
        this.minimumFreeNodeCountAfterGc,
        this.minimumFreeNodePercentageAfterGc,
        this.minimumNodeTableGrowth,
        this.minimumNodeTableSize,
        this.nodeTableBigThreshold,
        this.nodeTableSmallThreshold,
        this.useGlobalComposeCache,
        this.useShannonExists);
  }

  /**
   * Copy the current immutable object by setting a value for the {@link BddConfiguration#cacheComposeDivider() cacheComposeDivider} attribute.
   * A value equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for cacheComposeDivider
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableBddConfiguration withCacheComposeDivider(int value) {
    if (this.cacheComposeDivider == value) return this;
    return new ImmutableBddConfiguration(
        this.cacheBinaryBinsPerHash,
        this.cacheBinaryDivider,
        value,
        this.cacheSatisfactionBinsPerHash,
        this.cacheSatisfactionDivider,
        this.cacheTernaryBinsPerHash,
        this.cacheTernaryDivider,
        this.cacheUnaryBinsPerHash,
        this.cacheUnaryDivider,
        this.cacheVolatileBinsPerHash,
        this.cacheVolatileMultiplier,
        this.maximumNodeTableGrowth,
        this.minimumCacheSize,
        this.minimumDeadNodesCountForGcInGrow,
        this.minimumFreeNodeCountAfterGc,
        this.minimumFreeNodePercentageAfterGc,
        this.minimumNodeTableGrowth,
        this.minimumNodeTableSize,
        this.nodeTableBigThreshold,
        this.nodeTableSmallThreshold,
        this.useGlobalComposeCache,
        this.useShannonExists);
  }

  /**
   * Copy the current immutable object by setting a value for the {@link BddConfiguration#cacheSatisfactionBinsPerHash() cacheSatisfactionBinsPerHash} attribute.
   * A value equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for cacheSatisfactionBinsPerHash
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableBddConfiguration withCacheSatisfactionBinsPerHash(int value) {
    if (this.cacheSatisfactionBinsPerHash == value) return this;
    return new ImmutableBddConfiguration(
        this.cacheBinaryBinsPerHash,
        this.cacheBinaryDivider,
        this.cacheComposeDivider,
        value,
        this.cacheSatisfactionDivider,
        this.cacheTernaryBinsPerHash,
        this.cacheTernaryDivider,
        this.cacheUnaryBinsPerHash,
        this.cacheUnaryDivider,
        this.cacheVolatileBinsPerHash,
        this.cacheVolatileMultiplier,
        this.maximumNodeTableGrowth,
        this.minimumCacheSize,
        this.minimumDeadNodesCountForGcInGrow,
        this.minimumFreeNodeCountAfterGc,
        this.minimumFreeNodePercentageAfterGc,
        this.minimumNodeTableGrowth,
        this.minimumNodeTableSize,
        this.nodeTableBigThreshold,
        this.nodeTableSmallThreshold,
        this.useGlobalComposeCache,
        this.useShannonExists);
  }

  /**
   * Copy the current immutable object by setting a value for the {@link BddConfiguration#cacheSatisfactionDivider() cacheSatisfactionDivider} attribute.
   * A value equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for cacheSatisfactionDivider
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableBddConfiguration withCacheSatisfactionDivider(int value) {
    if (this.cacheSatisfactionDivider == value) return this;
    return new ImmutableBddConfiguration(
        this.cacheBinaryBinsPerHash,
        this.cacheBinaryDivider,
        this.cacheComposeDivider,
        this.cacheSatisfactionBinsPerHash,
        value,
        this.cacheTernaryBinsPerHash,
        this.cacheTernaryDivider,
        this.cacheUnaryBinsPerHash,
        this.cacheUnaryDivider,
        this.cacheVolatileBinsPerHash,
        this.cacheVolatileMultiplier,
        this.maximumNodeTableGrowth,
        this.minimumCacheSize,
        this.minimumDeadNodesCountForGcInGrow,
        this.minimumFreeNodeCountAfterGc,
        this.minimumFreeNodePercentageAfterGc,
        this.minimumNodeTableGrowth,
        this.minimumNodeTableSize,
        this.nodeTableBigThreshold,
        this.nodeTableSmallThreshold,
        this.useGlobalComposeCache,
        this.useShannonExists);
  }

  /**
   * Copy the current immutable object by setting a value for the {@link BddConfiguration#cacheTernaryBinsPerHash() cacheTernaryBinsPerHash} attribute.
   * A value equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for cacheTernaryBinsPerHash
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableBddConfiguration withCacheTernaryBinsPerHash(int value) {
    if (this.cacheTernaryBinsPerHash == value) return this;
    return new ImmutableBddConfiguration(
        this.cacheBinaryBinsPerHash,
        this.cacheBinaryDivider,
        this.cacheComposeDivider,
        this.cacheSatisfactionBinsPerHash,
        this.cacheSatisfactionDivider,
        value,
        this.cacheTernaryDivider,
        this.cacheUnaryBinsPerHash,
        this.cacheUnaryDivider,
        this.cacheVolatileBinsPerHash,
        this.cacheVolatileMultiplier,
        this.maximumNodeTableGrowth,
        this.minimumCacheSize,
        this.minimumDeadNodesCountForGcInGrow,
        this.minimumFreeNodeCountAfterGc,
        this.minimumFreeNodePercentageAfterGc,
        this.minimumNodeTableGrowth,
        this.minimumNodeTableSize,
        this.nodeTableBigThreshold,
        this.nodeTableSmallThreshold,
        this.useGlobalComposeCache,
        this.useShannonExists);
  }

  /**
   * Copy the current immutable object by setting a value for the {@link BddConfiguration#cacheTernaryDivider() cacheTernaryDivider} attribute.
   * A value equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for cacheTernaryDivider
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableBddConfiguration withCacheTernaryDivider(int value) {
    if (this.cacheTernaryDivider == value) return this;
    return new ImmutableBddConfiguration(
        this.cacheBinaryBinsPerHash,
        this.cacheBinaryDivider,
        this.cacheComposeDivider,
        this.cacheSatisfactionBinsPerHash,
        this.cacheSatisfactionDivider,
        this.cacheTernaryBinsPerHash,
        value,
        this.cacheUnaryBinsPerHash,
        this.cacheUnaryDivider,
        this.cacheVolatileBinsPerHash,
        this.cacheVolatileMultiplier,
        this.maximumNodeTableGrowth,
        this.minimumCacheSize,
        this.minimumDeadNodesCountForGcInGrow,
        this.minimumFreeNodeCountAfterGc,
        this.minimumFreeNodePercentageAfterGc,
        this.minimumNodeTableGrowth,
        this.minimumNodeTableSize,
        this.nodeTableBigThreshold,
        this.nodeTableSmallThreshold,
        this.useGlobalComposeCache,
        this.useShannonExists);
  }

  /**
   * Copy the current immutable object by setting a value for the {@link BddConfiguration#cacheUnaryBinsPerHash() cacheUnaryBinsPerHash} attribute.
   * A value equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for cacheUnaryBinsPerHash
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableBddConfiguration withCacheUnaryBinsPerHash(int value) {
    if (this.cacheUnaryBinsPerHash == value) return this;
    return new ImmutableBddConfiguration(
        this.cacheBinaryBinsPerHash,
        this.cacheBinaryDivider,
        this.cacheComposeDivider,
        this.cacheSatisfactionBinsPerHash,
        this.cacheSatisfactionDivider,
        this.cacheTernaryBinsPerHash,
        this.cacheTernaryDivider,
        value,
        this.cacheUnaryDivider,
        this.cacheVolatileBinsPerHash,
        this.cacheVolatileMultiplier,
        this.maximumNodeTableGrowth,
        this.minimumCacheSize,
        this.minimumDeadNodesCountForGcInGrow,
        this.minimumFreeNodeCountAfterGc,
        this.minimumFreeNodePercentageAfterGc,
        this.minimumNodeTableGrowth,
        this.minimumNodeTableSize,
        this.nodeTableBigThreshold,
        this.nodeTableSmallThreshold,
        this.useGlobalComposeCache,
        this.useShannonExists);
  }

  /**
   * Copy the current immutable object by setting a value for the {@link BddConfiguration#cacheUnaryDivider() cacheUnaryDivider} attribute.
   * A value equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for cacheUnaryDivider
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableBddConfiguration withCacheUnaryDivider(int value) {
    if (this.cacheUnaryDivider == value) return this;
    return new ImmutableBddConfiguration(
        this.cacheBinaryBinsPerHash,
        this.cacheBinaryDivider,
        this.cacheComposeDivider,
        this.cacheSatisfactionBinsPerHash,
        this.cacheSatisfactionDivider,
        this.cacheTernaryBinsPerHash,
        this.cacheTernaryDivider,
        this.cacheUnaryBinsPerHash,
        value,
        this.cacheVolatileBinsPerHash,
        this.cacheVolatileMultiplier,
        this.maximumNodeTableGrowth,
        this.minimumCacheSize,
        this.minimumDeadNodesCountForGcInGrow,
        this.minimumFreeNodeCountAfterGc,
        this.minimumFreeNodePercentageAfterGc,
        this.minimumNodeTableGrowth,
        this.minimumNodeTableSize,
        this.nodeTableBigThreshold,
        this.nodeTableSmallThreshold,
        this.useGlobalComposeCache,
        this.useShannonExists);
  }

  /**
   * Copy the current immutable object by setting a value for the {@link BddConfiguration#cacheVolatileBinsPerHash() cacheVolatileBinsPerHash} attribute.
   * A value equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for cacheVolatileBinsPerHash
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableBddConfiguration withCacheVolatileBinsPerHash(int value) {
    if (this.cacheVolatileBinsPerHash == value) return this;
    return new ImmutableBddConfiguration(
        this.cacheBinaryBinsPerHash,
        this.cacheBinaryDivider,
        this.cacheComposeDivider,
        this.cacheSatisfactionBinsPerHash,
        this.cacheSatisfactionDivider,
        this.cacheTernaryBinsPerHash,
        this.cacheTernaryDivider,
        this.cacheUnaryBinsPerHash,
        this.cacheUnaryDivider,
        value,
        this.cacheVolatileMultiplier,
        this.maximumNodeTableGrowth,
        this.minimumCacheSize,
        this.minimumDeadNodesCountForGcInGrow,
        this.minimumFreeNodeCountAfterGc,
        this.minimumFreeNodePercentageAfterGc,
        this.minimumNodeTableGrowth,
        this.minimumNodeTableSize,
        this.nodeTableBigThreshold,
        this.nodeTableSmallThreshold,
        this.useGlobalComposeCache,
        this.useShannonExists);
  }

  /**
   * Copy the current immutable object by setting a value for the {@link BddConfiguration#cacheVolatileMultiplier() cacheVolatileMultiplier} attribute.
   * A value equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for cacheVolatileMultiplier
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableBddConfiguration withCacheVolatileMultiplier(int value) {
    if (this.cacheVolatileMultiplier == value) return this;
    return new ImmutableBddConfiguration(
        this.cacheBinaryBinsPerHash,
        this.cacheBinaryDivider,
        this.cacheComposeDivider,
        this.cacheSatisfactionBinsPerHash,
        this.cacheSatisfactionDivider,
        this.cacheTernaryBinsPerHash,
        this.cacheTernaryDivider,
        this.cacheUnaryBinsPerHash,
        this.cacheUnaryDivider,
        this.cacheVolatileBinsPerHash,
        value,
        this.maximumNodeTableGrowth,
        this.minimumCacheSize,
        this.minimumDeadNodesCountForGcInGrow,
        this.minimumFreeNodeCountAfterGc,
        this.minimumFreeNodePercentageAfterGc,
        this.minimumNodeTableGrowth,
        this.minimumNodeTableSize,
        this.nodeTableBigThreshold,
        this.nodeTableSmallThreshold,
        this.useGlobalComposeCache,
        this.useShannonExists);
  }

  /**
   * Copy the current immutable object by setting a value for the {@link BddConfiguration#maximumNodeTableGrowth() maximumNodeTableGrowth} attribute.
   * A value equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for maximumNodeTableGrowth
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableBddConfiguration withMaximumNodeTableGrowth(int value) {
    if (this.maximumNodeTableGrowth == value) return this;
    return new ImmutableBddConfiguration(
        this.cacheBinaryBinsPerHash,
        this.cacheBinaryDivider,
        this.cacheComposeDivider,
        this.cacheSatisfactionBinsPerHash,
        this.cacheSatisfactionDivider,
        this.cacheTernaryBinsPerHash,
        this.cacheTernaryDivider,
        this.cacheUnaryBinsPerHash,
        this.cacheUnaryDivider,
        this.cacheVolatileBinsPerHash,
        this.cacheVolatileMultiplier,
        value,
        this.minimumCacheSize,
        this.minimumDeadNodesCountForGcInGrow,
        this.minimumFreeNodeCountAfterGc,
        this.minimumFreeNodePercentageAfterGc,
        this.minimumNodeTableGrowth,
        this.minimumNodeTableSize,
        this.nodeTableBigThreshold,
        this.nodeTableSmallThreshold,
        this.useGlobalComposeCache,
        this.useShannonExists);
  }

  /**
   * Copy the current immutable object by setting a value for the {@link BddConfiguration#minimumCacheSize() minimumCacheSize} attribute.
   * A value equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for minimumCacheSize
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableBddConfiguration withMinimumCacheSize(int value) {
    if (this.minimumCacheSize == value) return this;
    return new ImmutableBddConfiguration(
        this.cacheBinaryBinsPerHash,
        this.cacheBinaryDivider,
        this.cacheComposeDivider,
        this.cacheSatisfactionBinsPerHash,
        this.cacheSatisfactionDivider,
        this.cacheTernaryBinsPerHash,
        this.cacheTernaryDivider,
        this.cacheUnaryBinsPerHash,
        this.cacheUnaryDivider,
        this.cacheVolatileBinsPerHash,
        this.cacheVolatileMultiplier,
        this.maximumNodeTableGrowth,
        value,
        this.minimumDeadNodesCountForGcInGrow,
        this.minimumFreeNodeCountAfterGc,
        this.minimumFreeNodePercentageAfterGc,
        this.minimumNodeTableGrowth,
        this.minimumNodeTableSize,
        this.nodeTableBigThreshold,
        this.nodeTableSmallThreshold,
        this.useGlobalComposeCache,
        this.useShannonExists);
  }

  /**
   * Copy the current immutable object by setting a value for the {@link BddConfiguration#minimumDeadNodesCountForGcInGrow() minimumDeadNodesCountForGcInGrow} attribute.
   * A value equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for minimumDeadNodesCountForGcInGrow
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableBddConfiguration withMinimumDeadNodesCountForGcInGrow(int value) {
    if (this.minimumDeadNodesCountForGcInGrow == value) return this;
    return new ImmutableBddConfiguration(
        this.cacheBinaryBinsPerHash,
        this.cacheBinaryDivider,
        this.cacheComposeDivider,
        this.cacheSatisfactionBinsPerHash,
        this.cacheSatisfactionDivider,
        this.cacheTernaryBinsPerHash,
        this.cacheTernaryDivider,
        this.cacheUnaryBinsPerHash,
        this.cacheUnaryDivider,
        this.cacheVolatileBinsPerHash,
        this.cacheVolatileMultiplier,
        this.maximumNodeTableGrowth,
        this.minimumCacheSize,
        value,
        this.minimumFreeNodeCountAfterGc,
        this.minimumFreeNodePercentageAfterGc,
        this.minimumNodeTableGrowth,
        this.minimumNodeTableSize,
        this.nodeTableBigThreshold,
        this.nodeTableSmallThreshold,
        this.useGlobalComposeCache,
        this.useShannonExists);
  }

  /**
   * Copy the current immutable object by setting a value for the {@link BddConfiguration#minimumFreeNodeCountAfterGc() minimumFreeNodeCountAfterGc} attribute.
   * A value equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for minimumFreeNodeCountAfterGc
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableBddConfiguration withMinimumFreeNodeCountAfterGc(int value) {
    if (this.minimumFreeNodeCountAfterGc == value) return this;
    return new ImmutableBddConfiguration(
        this.cacheBinaryBinsPerHash,
        this.cacheBinaryDivider,
        this.cacheComposeDivider,
        this.cacheSatisfactionBinsPerHash,
        this.cacheSatisfactionDivider,
        this.cacheTernaryBinsPerHash,
        this.cacheTernaryDivider,
        this.cacheUnaryBinsPerHash,
        this.cacheUnaryDivider,
        this.cacheVolatileBinsPerHash,
        this.cacheVolatileMultiplier,
        this.maximumNodeTableGrowth,
        this.minimumCacheSize,
        this.minimumDeadNodesCountForGcInGrow,
        value,
        this.minimumFreeNodePercentageAfterGc,
        this.minimumNodeTableGrowth,
        this.minimumNodeTableSize,
        this.nodeTableBigThreshold,
        this.nodeTableSmallThreshold,
        this.useGlobalComposeCache,
        this.useShannonExists);
  }

  /**
   * Copy the current immutable object by setting a value for the {@link BddConfiguration#minimumFreeNodePercentageAfterGc() minimumFreeNodePercentageAfterGc} attribute.
   * A value strict bits equality used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for minimumFreeNodePercentageAfterGc
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableBddConfiguration withMinimumFreeNodePercentageAfterGc(float value) {
    if (Float.floatToIntBits(this.minimumFreeNodePercentageAfterGc) == Float.floatToIntBits(value)) return this;
    return new ImmutableBddConfiguration(
        this.cacheBinaryBinsPerHash,
        this.cacheBinaryDivider,
        this.cacheComposeDivider,
        this.cacheSatisfactionBinsPerHash,
        this.cacheSatisfactionDivider,
        this.cacheTernaryBinsPerHash,
        this.cacheTernaryDivider,
        this.cacheUnaryBinsPerHash,
        this.cacheUnaryDivider,
        this.cacheVolatileBinsPerHash,
        this.cacheVolatileMultiplier,
        this.maximumNodeTableGrowth,
        this.minimumCacheSize,
        this.minimumDeadNodesCountForGcInGrow,
        this.minimumFreeNodeCountAfterGc,
        value,
        this.minimumNodeTableGrowth,
        this.minimumNodeTableSize,
        this.nodeTableBigThreshold,
        this.nodeTableSmallThreshold,
        this.useGlobalComposeCache,
        this.useShannonExists);
  }

  /**
   * Copy the current immutable object by setting a value for the {@link BddConfiguration#minimumNodeTableGrowth() minimumNodeTableGrowth} attribute.
   * A value equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for minimumNodeTableGrowth
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableBddConfiguration withMinimumNodeTableGrowth(int value) {
    if (this.minimumNodeTableGrowth == value) return this;
    return new ImmutableBddConfiguration(
        this.cacheBinaryBinsPerHash,
        this.cacheBinaryDivider,
        this.cacheComposeDivider,
        this.cacheSatisfactionBinsPerHash,
        this.cacheSatisfactionDivider,
        this.cacheTernaryBinsPerHash,
        this.cacheTernaryDivider,
        this.cacheUnaryBinsPerHash,
        this.cacheUnaryDivider,
        this.cacheVolatileBinsPerHash,
        this.cacheVolatileMultiplier,
        this.maximumNodeTableGrowth,
        this.minimumCacheSize,
        this.minimumDeadNodesCountForGcInGrow,
        this.minimumFreeNodeCountAfterGc,
        this.minimumFreeNodePercentageAfterGc,
        value,
        this.minimumNodeTableSize,
        this.nodeTableBigThreshold,
        this.nodeTableSmallThreshold,
        this.useGlobalComposeCache,
        this.useShannonExists);
  }

  /**
   * Copy the current immutable object by setting a value for the {@link BddConfiguration#minimumNodeTableSize() minimumNodeTableSize} attribute.
   * A value equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for minimumNodeTableSize
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableBddConfiguration withMinimumNodeTableSize(int value) {
    if (this.minimumNodeTableSize == value) return this;
    return new ImmutableBddConfiguration(
        this.cacheBinaryBinsPerHash,
        this.cacheBinaryDivider,
        this.cacheComposeDivider,
        this.cacheSatisfactionBinsPerHash,
        this.cacheSatisfactionDivider,
        this.cacheTernaryBinsPerHash,
        this.cacheTernaryDivider,
        this.cacheUnaryBinsPerHash,
        this.cacheUnaryDivider,
        this.cacheVolatileBinsPerHash,
        this.cacheVolatileMultiplier,
        this.maximumNodeTableGrowth,
        this.minimumCacheSize,
        this.minimumDeadNodesCountForGcInGrow,
        this.minimumFreeNodeCountAfterGc,
        this.minimumFreeNodePercentageAfterGc,
        this.minimumNodeTableGrowth,
        value,
        this.nodeTableBigThreshold,
        this.nodeTableSmallThreshold,
        this.useGlobalComposeCache,
        this.useShannonExists);
  }

  /**
   * Copy the current immutable object by setting a value for the {@link BddConfiguration#nodeTableBigThreshold() nodeTableBigThreshold} attribute.
   * A value equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for nodeTableBigThreshold
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableBddConfiguration withNodeTableBigThreshold(int value) {
    if (this.nodeTableBigThreshold == value) return this;
    return new ImmutableBddConfiguration(
        this.cacheBinaryBinsPerHash,
        this.cacheBinaryDivider,
        this.cacheComposeDivider,
        this.cacheSatisfactionBinsPerHash,
        this.cacheSatisfactionDivider,
        this.cacheTernaryBinsPerHash,
        this.cacheTernaryDivider,
        this.cacheUnaryBinsPerHash,
        this.cacheUnaryDivider,
        this.cacheVolatileBinsPerHash,
        this.cacheVolatileMultiplier,
        this.maximumNodeTableGrowth,
        this.minimumCacheSize,
        this.minimumDeadNodesCountForGcInGrow,
        this.minimumFreeNodeCountAfterGc,
        this.minimumFreeNodePercentageAfterGc,
        this.minimumNodeTableGrowth,
        this.minimumNodeTableSize,
        value,
        this.nodeTableSmallThreshold,
        this.useGlobalComposeCache,
        this.useShannonExists);
  }

  /**
   * Copy the current immutable object by setting a value for the {@link BddConfiguration#nodeTableSmallThreshold() nodeTableSmallThreshold} attribute.
   * A value equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for nodeTableSmallThreshold
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableBddConfiguration withNodeTableSmallThreshold(int value) {
    if (this.nodeTableSmallThreshold == value) return this;
    return new ImmutableBddConfiguration(
        this.cacheBinaryBinsPerHash,
        this.cacheBinaryDivider,
        this.cacheComposeDivider,
        this.cacheSatisfactionBinsPerHash,
        this.cacheSatisfactionDivider,
        this.cacheTernaryBinsPerHash,
        this.cacheTernaryDivider,
        this.cacheUnaryBinsPerHash,
        this.cacheUnaryDivider,
        this.cacheVolatileBinsPerHash,
        this.cacheVolatileMultiplier,
        this.maximumNodeTableGrowth,
        this.minimumCacheSize,
        this.minimumDeadNodesCountForGcInGrow,
        this.minimumFreeNodeCountAfterGc,
        this.minimumFreeNodePercentageAfterGc,
        this.minimumNodeTableGrowth,
        this.minimumNodeTableSize,
        this.nodeTableBigThreshold,
        value,
        this.useGlobalComposeCache,
        this.useShannonExists);
  }

  /**
   * Copy the current immutable object by setting a value for the {@link BddConfiguration#useGlobalComposeCache() useGlobalComposeCache} attribute.
   * A value equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for useGlobalComposeCache
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableBddConfiguration withUseGlobalComposeCache(boolean value) {
    if (this.useGlobalComposeCache == value) return this;
    return new ImmutableBddConfiguration(
        this.cacheBinaryBinsPerHash,
        this.cacheBinaryDivider,
        this.cacheComposeDivider,
        this.cacheSatisfactionBinsPerHash,
        this.cacheSatisfactionDivider,
        this.cacheTernaryBinsPerHash,
        this.cacheTernaryDivider,
        this.cacheUnaryBinsPerHash,
        this.cacheUnaryDivider,
        this.cacheVolatileBinsPerHash,
        this.cacheVolatileMultiplier,
        this.maximumNodeTableGrowth,
        this.minimumCacheSize,
        this.minimumDeadNodesCountForGcInGrow,
        this.minimumFreeNodeCountAfterGc,
        this.minimumFreeNodePercentageAfterGc,
        this.minimumNodeTableGrowth,
        this.minimumNodeTableSize,
        this.nodeTableBigThreshold,
        this.nodeTableSmallThreshold,
        value,
        this.useShannonExists);
  }

  /**
   * Copy the current immutable object by setting a value for the {@link BddConfiguration#useShannonExists() useShannonExists} attribute.
   * A value equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for useShannonExists
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableBddConfiguration withUseShannonExists(boolean value) {
    if (this.useShannonExists == value) return this;
    return new ImmutableBddConfiguration(
        this.cacheBinaryBinsPerHash,
        this.cacheBinaryDivider,
        this.cacheComposeDivider,
        this.cacheSatisfactionBinsPerHash,
        this.cacheSatisfactionDivider,
        this.cacheTernaryBinsPerHash,
        this.cacheTernaryDivider,
        this.cacheUnaryBinsPerHash,
        this.cacheUnaryDivider,
        this.cacheVolatileBinsPerHash,
        this.cacheVolatileMultiplier,
        this.maximumNodeTableGrowth,
        this.minimumCacheSize,
        this.minimumDeadNodesCountForGcInGrow,
        this.minimumFreeNodeCountAfterGc,
        this.minimumFreeNodePercentageAfterGc,
        this.minimumNodeTableGrowth,
        this.minimumNodeTableSize,
        this.nodeTableBigThreshold,
        this.nodeTableSmallThreshold,
        this.useGlobalComposeCache,
        value);
  }

  /**
   * This instance is equal to all instances of {@code ImmutableBddConfiguration} that have equal attribute values.
   * @return {@code true} if {@code this} is equal to {@code another} instance
   */
  @Override
  public boolean equals(@Nullable Object another) {
    if (this == another) return true;
    return another instanceof ImmutableBddConfiguration
        && equalTo((ImmutableBddConfiguration) another);
  }

  private boolean equalTo(ImmutableBddConfiguration another) {
    return cacheBinaryBinsPerHash == another.cacheBinaryBinsPerHash
        && cacheBinaryDivider == another.cacheBinaryDivider
        && cacheComposeDivider == another.cacheComposeDivider
        && cacheSatisfactionBinsPerHash == another.cacheSatisfactionBinsPerHash
        && cacheSatisfactionDivider == another.cacheSatisfactionDivider
        && cacheTernaryBinsPerHash == another.cacheTernaryBinsPerHash
        && cacheTernaryDivider == another.cacheTernaryDivider
        && cacheUnaryBinsPerHash == another.cacheUnaryBinsPerHash
        && cacheUnaryDivider == another.cacheUnaryDivider
        && cacheVolatileBinsPerHash == another.cacheVolatileBinsPerHash
        && cacheVolatileMultiplier == another.cacheVolatileMultiplier
        && maximumNodeTableGrowth == another.maximumNodeTableGrowth
        && minimumCacheSize == another.minimumCacheSize
        && minimumDeadNodesCountForGcInGrow == another.minimumDeadNodesCountForGcInGrow
        && minimumFreeNodeCountAfterGc == another.minimumFreeNodeCountAfterGc
        && Float.floatToIntBits(minimumFreeNodePercentageAfterGc) == Float.floatToIntBits(another.minimumFreeNodePercentageAfterGc)
        && minimumNodeTableGrowth == another.minimumNodeTableGrowth
        && minimumNodeTableSize == another.minimumNodeTableSize
        && nodeTableBigThreshold == another.nodeTableBigThreshold
        && nodeTableSmallThreshold == another.nodeTableSmallThreshold
        && useGlobalComposeCache == another.useGlobalComposeCache
        && useShannonExists == another.useShannonExists;
  }

  /**
   * Computes a hash code from attributes: {@code cacheBinaryBinsPerHash}, {@code cacheBinaryDivider}, {@code cacheComposeDivider}, {@code cacheSatisfactionBinsPerHash}, {@code cacheSatisfactionDivider}, {@code cacheTernaryBinsPerHash}, {@code cacheTernaryDivider}, {@code cacheUnaryBinsPerHash}, {@code cacheUnaryDivider}, {@code cacheVolatileBinsPerHash}, {@code cacheVolatileMultiplier}, {@code maximumNodeTableGrowth}, {@code minimumCacheSize}, {@code minimumDeadNodesCountForGcInGrow}, {@code minimumFreeNodeCountAfterGc}, {@code minimumFreeNodePercentageAfterGc}, {@code minimumNodeTableGrowth}, {@code minimumNodeTableSize}, {@code nodeTableBigThreshold}, {@code nodeTableSmallThreshold}, {@code useGlobalComposeCache}, {@code useShannonExists}.
   * @return hashCode value
   */
  @Override
  public int hashCode() {
    int h = 31;
    h = h * 17 + cacheBinaryBinsPerHash;
    h = h * 17 + cacheBinaryDivider;
    h = h * 17 + cacheComposeDivider;
    h = h * 17 + cacheSatisfactionBinsPerHash;
    h = h * 17 + cacheSatisfactionDivider;
    h = h * 17 + cacheTernaryBinsPerHash;
    h = h * 17 + cacheTernaryDivider;
    h = h * 17 + cacheUnaryBinsPerHash;
    h = h * 17 + cacheUnaryDivider;
    h = h * 17 + cacheVolatileBinsPerHash;
    h = h * 17 + cacheVolatileMultiplier;
    h = h * 17 + maximumNodeTableGrowth;
    h = h * 17 + minimumCacheSize;
    h = h * 17 + minimumDeadNodesCountForGcInGrow;
    h = h * 17 + minimumFreeNodeCountAfterGc;
    h = h * 17 + Floats.hashCode(minimumFreeNodePercentageAfterGc);
    h = h * 17 + minimumNodeTableGrowth;
    h = h * 17 + minimumNodeTableSize;
    h = h * 17 + nodeTableBigThreshold;
    h = h * 17 + nodeTableSmallThreshold;
    h = h * 17 + Booleans.hashCode(useGlobalComposeCache);
    h = h * 17 + Booleans.hashCode(useShannonExists);
    return h;
  }

  /**
   * Prints the immutable value {@code BddConfiguration} with attribute values.
   * @return A string representation of the value
   */
  @Override
  public String toString() {
    return MoreObjects.toStringHelper("BddConfiguration")
        .omitNullValues()
        .add("cacheBinaryBinsPerHash", cacheBinaryBinsPerHash)
        .add("cacheBinaryDivider", cacheBinaryDivider)
        .add("cacheComposeDivider", cacheComposeDivider)
        .add("cacheSatisfactionBinsPerHash", cacheSatisfactionBinsPerHash)
        .add("cacheSatisfactionDivider", cacheSatisfactionDivider)
        .add("cacheTernaryBinsPerHash", cacheTernaryBinsPerHash)
        .add("cacheTernaryDivider", cacheTernaryDivider)
        .add("cacheUnaryBinsPerHash", cacheUnaryBinsPerHash)
        .add("cacheUnaryDivider", cacheUnaryDivider)
        .add("cacheVolatileBinsPerHash", cacheVolatileBinsPerHash)
        .add("cacheVolatileMultiplier", cacheVolatileMultiplier)
        .add("maximumNodeTableGrowth", maximumNodeTableGrowth)
        .add("minimumCacheSize", minimumCacheSize)
        .add("minimumDeadNodesCountForGcInGrow", minimumDeadNodesCountForGcInGrow)
        .add("minimumFreeNodeCountAfterGc", minimumFreeNodeCountAfterGc)
        .add("minimumFreeNodePercentageAfterGc", minimumFreeNodePercentageAfterGc)
        .add("minimumNodeTableGrowth", minimumNodeTableGrowth)
        .add("minimumNodeTableSize", minimumNodeTableSize)
        .add("nodeTableBigThreshold", nodeTableBigThreshold)
        .add("nodeTableSmallThreshold", nodeTableSmallThreshold)
        .add("useGlobalComposeCache", useGlobalComposeCache)
        .add("useShannonExists", useShannonExists)
        .toString();
  }

  /**
   * Creates an immutable copy of a {@link BddConfiguration} value.
   * Uses accessors to get values to initialize the new immutable instance.
   * If an instance is already immutable, it is returned as is.
   * @param instance The instance to copy
   * @return A copied immutable BddConfiguration instance
   */
  public static ImmutableBddConfiguration copyOf(BddConfiguration instance) {
    if (instance instanceof ImmutableBddConfiguration) {
      return (ImmutableBddConfiguration) instance;
    }
    return ImmutableBddConfiguration.builder()
        .from(instance)
        .build();
  }

  /**
   * Creates a builder for {@link ImmutableBddConfiguration ImmutableBddConfiguration}.
   * @return A new ImmutableBddConfiguration builder
   */
  public static ImmutableBddConfiguration.Builder builder() {
    return new ImmutableBddConfiguration.Builder();
  }

  /**
   * Builds instances of type {@link ImmutableBddConfiguration ImmutableBddConfiguration}.
   * Initialize attributes and then invoke the {@link #build()} method to create an
   * immutable instance.
   * <p><em>{@code Builder} is not thread-safe and generally should not be stored in a field or collection,
   * but instead used immediately to create instances.</em>
   */
  @NotThreadSafe
  public static final class Builder {
    private static final long OPT_BIT_CACHE_BINARY_BINS_PER_HASH = 0x1L;
    private static final long OPT_BIT_CACHE_BINARY_DIVIDER = 0x2L;
    private static final long OPT_BIT_CACHE_COMPOSE_DIVIDER = 0x4L;
    private static final long OPT_BIT_CACHE_SATISFACTION_BINS_PER_HASH = 0x8L;
    private static final long OPT_BIT_CACHE_SATISFACTION_DIVIDER = 0x10L;
    private static final long OPT_BIT_CACHE_TERNARY_BINS_PER_HASH = 0x20L;
    private static final long OPT_BIT_CACHE_TERNARY_DIVIDER = 0x40L;
    private static final long OPT_BIT_CACHE_UNARY_BINS_PER_HASH = 0x80L;
    private static final long OPT_BIT_CACHE_UNARY_DIVIDER = 0x100L;
    private static final long OPT_BIT_CACHE_VOLATILE_BINS_PER_HASH = 0x200L;
    private static final long OPT_BIT_CACHE_VOLATILE_MULTIPLIER = 0x400L;
    private static final long OPT_BIT_MAXIMUM_NODE_TABLE_GROWTH = 0x800L;
    private static final long OPT_BIT_MINIMUM_CACHE_SIZE = 0x1000L;
    private static final long OPT_BIT_MINIMUM_DEAD_NODES_COUNT_FOR_GC_IN_GROW = 0x2000L;
    private static final long OPT_BIT_MINIMUM_FREE_NODE_COUNT_AFTER_GC = 0x4000L;
    private static final long OPT_BIT_MINIMUM_FREE_NODE_PERCENTAGE_AFTER_GC = 0x8000L;
    private static final long OPT_BIT_MINIMUM_NODE_TABLE_GROWTH = 0x10000L;
    private static final long OPT_BIT_MINIMUM_NODE_TABLE_SIZE = 0x20000L;
    private static final long OPT_BIT_NODE_TABLE_BIG_THRESHOLD = 0x40000L;
    private static final long OPT_BIT_NODE_TABLE_SMALL_THRESHOLD = 0x80000L;
    private static final long OPT_BIT_USE_GLOBAL_COMPOSE_CACHE = 0x100000L;
    private static final long OPT_BIT_USE_SHANNON_EXISTS = 0x200000L;
    private long optBits;

    private int cacheBinaryBinsPerHash;
    private int cacheBinaryDivider;
    private int cacheComposeDivider;
    private int cacheSatisfactionBinsPerHash;
    private int cacheSatisfactionDivider;
    private int cacheTernaryBinsPerHash;
    private int cacheTernaryDivider;
    private int cacheUnaryBinsPerHash;
    private int cacheUnaryDivider;
    private int cacheVolatileBinsPerHash;
    private int cacheVolatileMultiplier;
    private int maximumNodeTableGrowth;
    private int minimumCacheSize;
    private int minimumDeadNodesCountForGcInGrow;
    private int minimumFreeNodeCountAfterGc;
    private float minimumFreeNodePercentageAfterGc;
    private int minimumNodeTableGrowth;
    private int minimumNodeTableSize;
    private int nodeTableBigThreshold;
    private int nodeTableSmallThreshold;
    private boolean useGlobalComposeCache;
    private boolean useShannonExists;

    private Builder() {
    }

    /**
     * Fill a builder with attribute values from the provided {@code BddConfiguration} instance.
     * Regular attribute values will be replaced with those from the given instance.
     * Absent optional values will not replace present values.
     * @param instance The instance from which to copy values
     * @return {@code this} builder for use in a chained invocation
     */
    public final Builder from(BddConfiguration instance) {
      Preconditions.checkNotNull(instance, "instance");
      cacheBinaryBinsPerHash(instance.cacheBinaryBinsPerHash());
      cacheBinaryDivider(instance.cacheBinaryDivider());
      cacheComposeDivider(instance.cacheComposeDivider());
      cacheSatisfactionBinsPerHash(instance.cacheSatisfactionBinsPerHash());
      cacheSatisfactionDivider(instance.cacheSatisfactionDivider());
      cacheTernaryBinsPerHash(instance.cacheTernaryBinsPerHash());
      cacheTernaryDivider(instance.cacheTernaryDivider());
      cacheUnaryBinsPerHash(instance.cacheUnaryBinsPerHash());
      cacheUnaryDivider(instance.cacheUnaryDivider());
      cacheVolatileBinsPerHash(instance.cacheVolatileBinsPerHash());
      cacheVolatileMultiplier(instance.cacheVolatileMultiplier());
      maximumNodeTableGrowth(instance.maximumNodeTableGrowth());
      minimumCacheSize(instance.minimumCacheSize());
      minimumDeadNodesCountForGcInGrow(instance.minimumDeadNodesCountForGcInGrow());
      minimumFreeNodeCountAfterGc(instance.minimumFreeNodeCountAfterGc());
      minimumFreeNodePercentageAfterGc(instance.minimumFreeNodePercentageAfterGc());
      minimumNodeTableGrowth(instance.minimumNodeTableGrowth());
      minimumNodeTableSize(instance.minimumNodeTableSize());
      nodeTableBigThreshold(instance.nodeTableBigThreshold());
      nodeTableSmallThreshold(instance.nodeTableSmallThreshold());
      useGlobalComposeCache(instance.useGlobalComposeCache());
      useShannonExists(instance.useShannonExists());
      return this;
    }

    /**
     * Initializes the value for the {@link BddConfiguration#cacheBinaryBinsPerHash() cacheBinaryBinsPerHash} attribute.
     * <p><em>If not set, this attribute will have a default value as returned by the initializer of {@link BddConfiguration#cacheBinaryBinsPerHash() cacheBinaryBinsPerHash}.</em>
     * @param cacheBinaryBinsPerHash The value for cacheBinaryBinsPerHash 
     * @return {@code this} builder for use in a chained invocation
     */
    public final Builder cacheBinaryBinsPerHash(int cacheBinaryBinsPerHash) {
      this.cacheBinaryBinsPerHash = cacheBinaryBinsPerHash;
      optBits |= OPT_BIT_CACHE_BINARY_BINS_PER_HASH;
      return this;
    }

    /**
     * Initializes the value for the {@link BddConfiguration#cacheBinaryDivider() cacheBinaryDivider} attribute.
     * <p><em>If not set, this attribute will have a default value as returned by the initializer of {@link BddConfiguration#cacheBinaryDivider() cacheBinaryDivider}.</em>
     * @param cacheBinaryDivider The value for cacheBinaryDivider 
     * @return {@code this} builder for use in a chained invocation
     */
    public final Builder cacheBinaryDivider(int cacheBinaryDivider) {
      this.cacheBinaryDivider = cacheBinaryDivider;
      optBits |= OPT_BIT_CACHE_BINARY_DIVIDER;
      return this;
    }

    /**
     * Initializes the value for the {@link BddConfiguration#cacheComposeDivider() cacheComposeDivider} attribute.
     * <p><em>If not set, this attribute will have a default value as returned by the initializer of {@link BddConfiguration#cacheComposeDivider() cacheComposeDivider}.</em>
     * @param cacheComposeDivider The value for cacheComposeDivider 
     * @return {@code this} builder for use in a chained invocation
     */
    public final Builder cacheComposeDivider(int cacheComposeDivider) {
      this.cacheComposeDivider = cacheComposeDivider;
      optBits |= OPT_BIT_CACHE_COMPOSE_DIVIDER;
      return this;
    }

    /**
     * Initializes the value for the {@link BddConfiguration#cacheSatisfactionBinsPerHash() cacheSatisfactionBinsPerHash} attribute.
     * <p><em>If not set, this attribute will have a default value as returned by the initializer of {@link BddConfiguration#cacheSatisfactionBinsPerHash() cacheSatisfactionBinsPerHash}.</em>
     * @param cacheSatisfactionBinsPerHash The value for cacheSatisfactionBinsPerHash 
     * @return {@code this} builder for use in a chained invocation
     */
    public final Builder cacheSatisfactionBinsPerHash(int cacheSatisfactionBinsPerHash) {
      this.cacheSatisfactionBinsPerHash = cacheSatisfactionBinsPerHash;
      optBits |= OPT_BIT_CACHE_SATISFACTION_BINS_PER_HASH;
      return this;
    }

    /**
     * Initializes the value for the {@link BddConfiguration#cacheSatisfactionDivider() cacheSatisfactionDivider} attribute.
     * <p><em>If not set, this attribute will have a default value as returned by the initializer of {@link BddConfiguration#cacheSatisfactionDivider() cacheSatisfactionDivider}.</em>
     * @param cacheSatisfactionDivider The value for cacheSatisfactionDivider 
     * @return {@code this} builder for use in a chained invocation
     */
    public final Builder cacheSatisfactionDivider(int cacheSatisfactionDivider) {
      this.cacheSatisfactionDivider = cacheSatisfactionDivider;
      optBits |= OPT_BIT_CACHE_SATISFACTION_DIVIDER;
      return this;
    }

    /**
     * Initializes the value for the {@link BddConfiguration#cacheTernaryBinsPerHash() cacheTernaryBinsPerHash} attribute.
     * <p><em>If not set, this attribute will have a default value as returned by the initializer of {@link BddConfiguration#cacheTernaryBinsPerHash() cacheTernaryBinsPerHash}.</em>
     * @param cacheTernaryBinsPerHash The value for cacheTernaryBinsPerHash 
     * @return {@code this} builder for use in a chained invocation
     */
    public final Builder cacheTernaryBinsPerHash(int cacheTernaryBinsPerHash) {
      this.cacheTernaryBinsPerHash = cacheTernaryBinsPerHash;
      optBits |= OPT_BIT_CACHE_TERNARY_BINS_PER_HASH;
      return this;
    }

    /**
     * Initializes the value for the {@link BddConfiguration#cacheTernaryDivider() cacheTernaryDivider} attribute.
     * <p><em>If not set, this attribute will have a default value as returned by the initializer of {@link BddConfiguration#cacheTernaryDivider() cacheTernaryDivider}.</em>
     * @param cacheTernaryDivider The value for cacheTernaryDivider 
     * @return {@code this} builder for use in a chained invocation
     */
    public final Builder cacheTernaryDivider(int cacheTernaryDivider) {
      this.cacheTernaryDivider = cacheTernaryDivider;
      optBits |= OPT_BIT_CACHE_TERNARY_DIVIDER;
      return this;
    }

    /**
     * Initializes the value for the {@link BddConfiguration#cacheUnaryBinsPerHash() cacheUnaryBinsPerHash} attribute.
     * <p><em>If not set, this attribute will have a default value as returned by the initializer of {@link BddConfiguration#cacheUnaryBinsPerHash() cacheUnaryBinsPerHash}.</em>
     * @param cacheUnaryBinsPerHash The value for cacheUnaryBinsPerHash 
     * @return {@code this} builder for use in a chained invocation
     */
    public final Builder cacheUnaryBinsPerHash(int cacheUnaryBinsPerHash) {
      this.cacheUnaryBinsPerHash = cacheUnaryBinsPerHash;
      optBits |= OPT_BIT_CACHE_UNARY_BINS_PER_HASH;
      return this;
    }

    /**
     * Initializes the value for the {@link BddConfiguration#cacheUnaryDivider() cacheUnaryDivider} attribute.
     * <p><em>If not set, this attribute will have a default value as returned by the initializer of {@link BddConfiguration#cacheUnaryDivider() cacheUnaryDivider}.</em>
     * @param cacheUnaryDivider The value for cacheUnaryDivider 
     * @return {@code this} builder for use in a chained invocation
     */
    public final Builder cacheUnaryDivider(int cacheUnaryDivider) {
      this.cacheUnaryDivider = cacheUnaryDivider;
      optBits |= OPT_BIT_CACHE_UNARY_DIVIDER;
      return this;
    }

    /**
     * Initializes the value for the {@link BddConfiguration#cacheVolatileBinsPerHash() cacheVolatileBinsPerHash} attribute.
     * <p><em>If not set, this attribute will have a default value as returned by the initializer of {@link BddConfiguration#cacheVolatileBinsPerHash() cacheVolatileBinsPerHash}.</em>
     * @param cacheVolatileBinsPerHash The value for cacheVolatileBinsPerHash 
     * @return {@code this} builder for use in a chained invocation
     */
    public final Builder cacheVolatileBinsPerHash(int cacheVolatileBinsPerHash) {
      this.cacheVolatileBinsPerHash = cacheVolatileBinsPerHash;
      optBits |= OPT_BIT_CACHE_VOLATILE_BINS_PER_HASH;
      return this;
    }

    /**
     * Initializes the value for the {@link BddConfiguration#cacheVolatileMultiplier() cacheVolatileMultiplier} attribute.
     * <p><em>If not set, this attribute will have a default value as returned by the initializer of {@link BddConfiguration#cacheVolatileMultiplier() cacheVolatileMultiplier}.</em>
     * @param cacheVolatileMultiplier The value for cacheVolatileMultiplier 
     * @return {@code this} builder for use in a chained invocation
     */
    public final Builder cacheVolatileMultiplier(int cacheVolatileMultiplier) {
      this.cacheVolatileMultiplier = cacheVolatileMultiplier;
      optBits |= OPT_BIT_CACHE_VOLATILE_MULTIPLIER;
      return this;
    }

    /**
     * Initializes the value for the {@link BddConfiguration#maximumNodeTableGrowth() maximumNodeTableGrowth} attribute.
     * <p><em>If not set, this attribute will have a default value as returned by the initializer of {@link BddConfiguration#maximumNodeTableGrowth() maximumNodeTableGrowth}.</em>
     * @param maximumNodeTableGrowth The value for maximumNodeTableGrowth 
     * @return {@code this} builder for use in a chained invocation
     */
    public final Builder maximumNodeTableGrowth(int maximumNodeTableGrowth) {
      this.maximumNodeTableGrowth = maximumNodeTableGrowth;
      optBits |= OPT_BIT_MAXIMUM_NODE_TABLE_GROWTH;
      return this;
    }

    /**
     * Initializes the value for the {@link BddConfiguration#minimumCacheSize() minimumCacheSize} attribute.
     * <p><em>If not set, this attribute will have a default value as returned by the initializer of {@link BddConfiguration#minimumCacheSize() minimumCacheSize}.</em>
     * @param minimumCacheSize The value for minimumCacheSize 
     * @return {@code this} builder for use in a chained invocation
     */
    public final Builder minimumCacheSize(int minimumCacheSize) {
      this.minimumCacheSize = minimumCacheSize;
      optBits |= OPT_BIT_MINIMUM_CACHE_SIZE;
      return this;
    }

    /**
     * Initializes the value for the {@link BddConfiguration#minimumDeadNodesCountForGcInGrow() minimumDeadNodesCountForGcInGrow} attribute.
     * <p><em>If not set, this attribute will have a default value as returned by the initializer of {@link BddConfiguration#minimumDeadNodesCountForGcInGrow() minimumDeadNodesCountForGcInGrow}.</em>
     * @param minimumDeadNodesCountForGcInGrow The value for minimumDeadNodesCountForGcInGrow 
     * @return {@code this} builder for use in a chained invocation
     */
    public final Builder minimumDeadNodesCountForGcInGrow(int minimumDeadNodesCountForGcInGrow) {
      this.minimumDeadNodesCountForGcInGrow = minimumDeadNodesCountForGcInGrow;
      optBits |= OPT_BIT_MINIMUM_DEAD_NODES_COUNT_FOR_GC_IN_GROW;
      return this;
    }

    /**
     * Initializes the value for the {@link BddConfiguration#minimumFreeNodeCountAfterGc() minimumFreeNodeCountAfterGc} attribute.
     * <p><em>If not set, this attribute will have a default value as returned by the initializer of {@link BddConfiguration#minimumFreeNodeCountAfterGc() minimumFreeNodeCountAfterGc}.</em>
     * @param minimumFreeNodeCountAfterGc The value for minimumFreeNodeCountAfterGc 
     * @return {@code this} builder for use in a chained invocation
     */
    public final Builder minimumFreeNodeCountAfterGc(int minimumFreeNodeCountAfterGc) {
      this.minimumFreeNodeCountAfterGc = minimumFreeNodeCountAfterGc;
      optBits |= OPT_BIT_MINIMUM_FREE_NODE_COUNT_AFTER_GC;
      return this;
    }

    /**
     * Initializes the value for the {@link BddConfiguration#minimumFreeNodePercentageAfterGc() minimumFreeNodePercentageAfterGc} attribute.
     * <p><em>If not set, this attribute will have a default value as returned by the initializer of {@link BddConfiguration#minimumFreeNodePercentageAfterGc() minimumFreeNodePercentageAfterGc}.</em>
     * @param minimumFreeNodePercentageAfterGc The value for minimumFreeNodePercentageAfterGc 
     * @return {@code this} builder for use in a chained invocation
     */
    public final Builder minimumFreeNodePercentageAfterGc(float minimumFreeNodePercentageAfterGc) {
      this.minimumFreeNodePercentageAfterGc = minimumFreeNodePercentageAfterGc;
      optBits |= OPT_BIT_MINIMUM_FREE_NODE_PERCENTAGE_AFTER_GC;
      return this;
    }

    /**
     * Initializes the value for the {@link BddConfiguration#minimumNodeTableGrowth() minimumNodeTableGrowth} attribute.
     * <p><em>If not set, this attribute will have a default value as returned by the initializer of {@link BddConfiguration#minimumNodeTableGrowth() minimumNodeTableGrowth}.</em>
     * @param minimumNodeTableGrowth The value for minimumNodeTableGrowth 
     * @return {@code this} builder for use in a chained invocation
     */
    public final Builder minimumNodeTableGrowth(int minimumNodeTableGrowth) {
      this.minimumNodeTableGrowth = minimumNodeTableGrowth;
      optBits |= OPT_BIT_MINIMUM_NODE_TABLE_GROWTH;
      return this;
    }

    /**
     * Initializes the value for the {@link BddConfiguration#minimumNodeTableSize() minimumNodeTableSize} attribute.
     * <p><em>If not set, this attribute will have a default value as returned by the initializer of {@link BddConfiguration#minimumNodeTableSize() minimumNodeTableSize}.</em>
     * @param minimumNodeTableSize The value for minimumNodeTableSize 
     * @return {@code this} builder for use in a chained invocation
     */
    public final Builder minimumNodeTableSize(int minimumNodeTableSize) {
      this.minimumNodeTableSize = minimumNodeTableSize;
      optBits |= OPT_BIT_MINIMUM_NODE_TABLE_SIZE;
      return this;
    }

    /**
     * Initializes the value for the {@link BddConfiguration#nodeTableBigThreshold() nodeTableBigThreshold} attribute.
     * <p><em>If not set, this attribute will have a default value as returned by the initializer of {@link BddConfiguration#nodeTableBigThreshold() nodeTableBigThreshold}.</em>
     * @param nodeTableBigThreshold The value for nodeTableBigThreshold 
     * @return {@code this} builder for use in a chained invocation
     */
    public final Builder nodeTableBigThreshold(int nodeTableBigThreshold) {
      this.nodeTableBigThreshold = nodeTableBigThreshold;
      optBits |= OPT_BIT_NODE_TABLE_BIG_THRESHOLD;
      return this;
    }

    /**
     * Initializes the value for the {@link BddConfiguration#nodeTableSmallThreshold() nodeTableSmallThreshold} attribute.
     * <p><em>If not set, this attribute will have a default value as returned by the initializer of {@link BddConfiguration#nodeTableSmallThreshold() nodeTableSmallThreshold}.</em>
     * @param nodeTableSmallThreshold The value for nodeTableSmallThreshold 
     * @return {@code this} builder for use in a chained invocation
     */
    public final Builder nodeTableSmallThreshold(int nodeTableSmallThreshold) {
      this.nodeTableSmallThreshold = nodeTableSmallThreshold;
      optBits |= OPT_BIT_NODE_TABLE_SMALL_THRESHOLD;
      return this;
    }

    /**
     * Initializes the value for the {@link BddConfiguration#useGlobalComposeCache() useGlobalComposeCache} attribute.
     * <p><em>If not set, this attribute will have a default value as returned by the initializer of {@link BddConfiguration#useGlobalComposeCache() useGlobalComposeCache}.</em>
     * @param useGlobalComposeCache The value for useGlobalComposeCache 
     * @return {@code this} builder for use in a chained invocation
     */
    public final Builder useGlobalComposeCache(boolean useGlobalComposeCache) {
      this.useGlobalComposeCache = useGlobalComposeCache;
      optBits |= OPT_BIT_USE_GLOBAL_COMPOSE_CACHE;
      return this;
    }

    /**
     * Initializes the value for the {@link BddConfiguration#useShannonExists() useShannonExists} attribute.
     * <p><em>If not set, this attribute will have a default value as returned by the initializer of {@link BddConfiguration#useShannonExists() useShannonExists}.</em>
     * @param useShannonExists The value for useShannonExists 
     * @return {@code this} builder for use in a chained invocation
     */
    public final Builder useShannonExists(boolean useShannonExists) {
      this.useShannonExists = useShannonExists;
      optBits |= OPT_BIT_USE_SHANNON_EXISTS;
      return this;
    }

    /**
     * Builds a new {@link ImmutableBddConfiguration ImmutableBddConfiguration}.
     * @return An immutable instance of BddConfiguration
     * @throws java.lang.IllegalStateException if any required attributes are missing
     */
    public ImmutableBddConfiguration build() {
      return new ImmutableBddConfiguration(this);
    }

    private boolean cacheBinaryBinsPerHashIsSet() {
      return (optBits & OPT_BIT_CACHE_BINARY_BINS_PER_HASH) != 0;
    }

    private boolean cacheBinaryDividerIsSet() {
      return (optBits & OPT_BIT_CACHE_BINARY_DIVIDER) != 0;
    }

    private boolean cacheComposeDividerIsSet() {
      return (optBits & OPT_BIT_CACHE_COMPOSE_DIVIDER) != 0;
    }

    private boolean cacheSatisfactionBinsPerHashIsSet() {
      return (optBits & OPT_BIT_CACHE_SATISFACTION_BINS_PER_HASH) != 0;
    }

    private boolean cacheSatisfactionDividerIsSet() {
      return (optBits & OPT_BIT_CACHE_SATISFACTION_DIVIDER) != 0;
    }

    private boolean cacheTernaryBinsPerHashIsSet() {
      return (optBits & OPT_BIT_CACHE_TERNARY_BINS_PER_HASH) != 0;
    }

    private boolean cacheTernaryDividerIsSet() {
      return (optBits & OPT_BIT_CACHE_TERNARY_DIVIDER) != 0;
    }

    private boolean cacheUnaryBinsPerHashIsSet() {
      return (optBits & OPT_BIT_CACHE_UNARY_BINS_PER_HASH) != 0;
    }

    private boolean cacheUnaryDividerIsSet() {
      return (optBits & OPT_BIT_CACHE_UNARY_DIVIDER) != 0;
    }

    private boolean cacheVolatileBinsPerHashIsSet() {
      return (optBits & OPT_BIT_CACHE_VOLATILE_BINS_PER_HASH) != 0;
    }

    private boolean cacheVolatileMultiplierIsSet() {
      return (optBits & OPT_BIT_CACHE_VOLATILE_MULTIPLIER) != 0;
    }

    private boolean maximumNodeTableGrowthIsSet() {
      return (optBits & OPT_BIT_MAXIMUM_NODE_TABLE_GROWTH) != 0;
    }

    private boolean minimumCacheSizeIsSet() {
      return (optBits & OPT_BIT_MINIMUM_CACHE_SIZE) != 0;
    }

    private boolean minimumDeadNodesCountForGcInGrowIsSet() {
      return (optBits & OPT_BIT_MINIMUM_DEAD_NODES_COUNT_FOR_GC_IN_GROW) != 0;
    }

    private boolean minimumFreeNodeCountAfterGcIsSet() {
      return (optBits & OPT_BIT_MINIMUM_FREE_NODE_COUNT_AFTER_GC) != 0;
    }

    private boolean minimumFreeNodePercentageAfterGcIsSet() {
      return (optBits & OPT_BIT_MINIMUM_FREE_NODE_PERCENTAGE_AFTER_GC) != 0;
    }

    private boolean minimumNodeTableGrowthIsSet() {
      return (optBits & OPT_BIT_MINIMUM_NODE_TABLE_GROWTH) != 0;
    }

    private boolean minimumNodeTableSizeIsSet() {
      return (optBits & OPT_BIT_MINIMUM_NODE_TABLE_SIZE) != 0;
    }

    private boolean nodeTableBigThresholdIsSet() {
      return (optBits & OPT_BIT_NODE_TABLE_BIG_THRESHOLD) != 0;
    }

    private boolean nodeTableSmallThresholdIsSet() {
      return (optBits & OPT_BIT_NODE_TABLE_SMALL_THRESHOLD) != 0;
    }

    private boolean useGlobalComposeCacheIsSet() {
      return (optBits & OPT_BIT_USE_GLOBAL_COMPOSE_CACHE) != 0;
    }

    private boolean useShannonExistsIsSet() {
      return (optBits & OPT_BIT_USE_SHANNON_EXISTS) != 0;
    }
  }
}
