package owl.ltl.tlsf;

import com.google.common.base.MoreObjects;
import com.google.common.base.Preconditions;
import com.google.common.collect.ImmutableBiMap;
import com.google.common.collect.Lists;
import edu.umd.cs.findbugs.annotations.SuppressFBWarnings;
import java.util.ArrayList;
import java.util.BitSet;
import java.util.List;
import java.util.Map;
import javax.annotation.Generated;
import javax.annotation.Nullable;
import javax.annotation.ParametersAreNonnullByDefault;
import javax.annotation.concurrent.Immutable;
import javax.annotation.concurrent.NotThreadSafe;
import owl.ltl.Formula;

/**
 * Immutable implementation of {@link TLSF}.
 * <p>
 * Use the builder to create immutable instances:
 * {@code ImmutableTLSF.builder()}.
 */
@SuppressWarnings({"all"})
@SuppressFBWarnings
@ParametersAreNonnullByDefault
@Generated({"Immutables.generator", "TLSF"})
@Immutable
public final class ImmutableTLSF extends TLSF {
  private final Formula assert_;
  private final Formula assume;
  private final String description;
  private final Formula guarantee;
  private final Formula initially;
  private final BitSet inputs;
  private final ImmutableBiMap<String, Integer> mapping;
  private final BitSet outputs;
  private final Formula preset;
  private final Formula require;
  private final TLSF.Semantics semantics;
  private final TLSF.Semantics target;
  private final String title;
  private final Formula toFormula;

  private ImmutableTLSF(ImmutableTLSF.Builder builder) {
    this.description = builder.description;
    this.inputs = builder.inputs;
    this.mapping = builder.mapping.build();
    this.outputs = builder.outputs;
    this.semantics = builder.semantics;
    this.target = builder.target;
    this.title = builder.title;
    if (builder.assert_ != null) {
      initShim.assert_(builder.assert_);
    }
    if (builder.assume != null) {
      initShim.assume(builder.assume);
    }
    if (builder.guarantee != null) {
      initShim.guarantee(builder.guarantee);
    }
    if (builder.initially != null) {
      initShim.initially(builder.initially);
    }
    if (builder.preset != null) {
      initShim.preset(builder.preset);
    }
    if (builder.require != null) {
      initShim.require(builder.require);
    }
    this.assert_ = initShim.assert_();
    this.assume = initShim.assume();
    this.guarantee = initShim.guarantee();
    this.initially = initShim.initially();
    this.preset = initShim.preset();
    this.require = initShim.require();
    this.toFormula = initShim.toFormula();
    this.initShim = null;
  }

  private ImmutableTLSF(
      Formula assert_,
      Formula assume,
      String description,
      Formula guarantee,
      Formula initially,
      BitSet inputs,
      ImmutableBiMap<String, Integer> mapping,
      BitSet outputs,
      Formula preset,
      Formula require,
      TLSF.Semantics semantics,
      TLSF.Semantics target,
      String title) {
    this.assert_ = assert_;
    this.assume = assume;
    this.description = description;
    this.guarantee = guarantee;
    this.initially = initially;
    this.inputs = inputs;
    this.mapping = mapping;
    this.outputs = outputs;
    this.preset = preset;
    this.require = require;
    this.semantics = semantics;
    this.target = target;
    this.title = title;
    initShim.assert_(this.assert_);
    initShim.assume(this.assume);
    initShim.guarantee(this.guarantee);
    initShim.initially(this.initially);
    initShim.preset(this.preset);
    initShim.require(this.require);
    this.toFormula = initShim.toFormula();
    this.initShim = null;
  }

  private static final int STAGE_INITIALIZING = -1;
  private static final int STAGE_UNINITIALIZED = 0;
  private static final int STAGE_INITIALIZED = 1;
  private transient volatile InitShim initShim = new InitShim();

  private final class InitShim {
    private Formula assert_;
    private int assert_BuildStage;

    Formula assert_() {
      if (assert_BuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (assert_BuildStage == STAGE_UNINITIALIZED) {
        assert_BuildStage = STAGE_INITIALIZING;
        this.assert_ = Preconditions.checkNotNull(ImmutableTLSF.super.assert_(), "assert_");
        assert_BuildStage = STAGE_INITIALIZED;
      }
      return this.assert_;
    }

    void assert_(Formula assert_) {
      this.assert_ = assert_;
      assert_BuildStage = STAGE_INITIALIZED;
    }
    private Formula assume;
    private int assumeBuildStage;

    Formula assume() {
      if (assumeBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (assumeBuildStage == STAGE_UNINITIALIZED) {
        assumeBuildStage = STAGE_INITIALIZING;
        this.assume = Preconditions.checkNotNull(ImmutableTLSF.super.assume(), "assume");
        assumeBuildStage = STAGE_INITIALIZED;
      }
      return this.assume;
    }

    void assume(Formula assume) {
      this.assume = assume;
      assumeBuildStage = STAGE_INITIALIZED;
    }
    private Formula guarantee;
    private int guaranteeBuildStage;

    Formula guarantee() {
      if (guaranteeBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (guaranteeBuildStage == STAGE_UNINITIALIZED) {
        guaranteeBuildStage = STAGE_INITIALIZING;
        this.guarantee = Preconditions.checkNotNull(ImmutableTLSF.super.guarantee(), "guarantee");
        guaranteeBuildStage = STAGE_INITIALIZED;
      }
      return this.guarantee;
    }

    void guarantee(Formula guarantee) {
      this.guarantee = guarantee;
      guaranteeBuildStage = STAGE_INITIALIZED;
    }
    private Formula initially;
    private int initiallyBuildStage;

    Formula initially() {
      if (initiallyBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (initiallyBuildStage == STAGE_UNINITIALIZED) {
        initiallyBuildStage = STAGE_INITIALIZING;
        this.initially = Preconditions.checkNotNull(ImmutableTLSF.super.initially(), "initially");
        initiallyBuildStage = STAGE_INITIALIZED;
      }
      return this.initially;
    }

    void initially(Formula initially) {
      this.initially = initially;
      initiallyBuildStage = STAGE_INITIALIZED;
    }
    private Formula preset;
    private int presetBuildStage;

    Formula preset() {
      if (presetBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (presetBuildStage == STAGE_UNINITIALIZED) {
        presetBuildStage = STAGE_INITIALIZING;
        this.preset = Preconditions.checkNotNull(ImmutableTLSF.super.preset(), "preset");
        presetBuildStage = STAGE_INITIALIZED;
      }
      return this.preset;
    }

    void preset(Formula preset) {
      this.preset = preset;
      presetBuildStage = STAGE_INITIALIZED;
    }
    private Formula require;
    private int requireBuildStage;

    Formula require() {
      if (requireBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (requireBuildStage == STAGE_UNINITIALIZED) {
        requireBuildStage = STAGE_INITIALIZING;
        this.require = Preconditions.checkNotNull(ImmutableTLSF.super.require(), "require");
        requireBuildStage = STAGE_INITIALIZED;
      }
      return this.require;
    }

    void require(Formula require) {
      this.require = require;
      requireBuildStage = STAGE_INITIALIZED;
    }
    private Formula toFormula;
    private int toFormulaBuildStage;

    Formula toFormula() {
      if (toFormulaBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (toFormulaBuildStage == STAGE_UNINITIALIZED) {
        toFormulaBuildStage = STAGE_INITIALIZING;
        this.toFormula = Preconditions.checkNotNull(ImmutableTLSF.super.toFormula(), "toFormula");
        toFormulaBuildStage = STAGE_INITIALIZED;
      }
      return this.toFormula;
    }

    private String formatInitCycleMessage() {
      ArrayList<String> attributes = Lists.newArrayList();
      if (assert_BuildStage == STAGE_INITIALIZING) attributes.add("assert_");
      if (assumeBuildStage == STAGE_INITIALIZING) attributes.add("assume");
      if (guaranteeBuildStage == STAGE_INITIALIZING) attributes.add("guarantee");
      if (initiallyBuildStage == STAGE_INITIALIZING) attributes.add("initially");
      if (presetBuildStage == STAGE_INITIALIZING) attributes.add("preset");
      if (requireBuildStage == STAGE_INITIALIZING) attributes.add("require");
      if (toFormulaBuildStage == STAGE_INITIALIZING) attributes.add("toFormula");
      return "Cannot build TLSF, attribute initializers form cycle" + attributes;
    }
  }

  /**
   * @return The value of the {@code assert_} attribute
   */
  @Override
  public Formula assert_() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.assert_()
        : this.assert_;
  }

  /**
   * @return The value of the {@code assume} attribute
   */
  @Override
  public Formula assume() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.assume()
        : this.assume;
  }

  /**
   * @return The value of the {@code description} attribute
   */
  @Override
  public String description() {
    return description;
  }

  /**
   * @return The value of the {@code guarantee} attribute
   */
  @Override
  public Formula guarantee() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.guarantee()
        : this.guarantee;
  }

  /**
   * @return The value of the {@code initially} attribute
   */
  @Override
  public Formula initially() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.initially()
        : this.initially;
  }

  /**
   * @return The value of the {@code inputs} attribute
   */
  @Override
  public BitSet inputs() {
    return inputs;
  }

  /**
   * @return The value of the {@code mapping} attribute
   */
  @Override
  public ImmutableBiMap<String, Integer> mapping() {
    return mapping;
  }

  /**
   * @return The value of the {@code outputs} attribute
   */
  @Override
  public BitSet outputs() {
    return outputs;
  }

  /**
   * @return The value of the {@code preset} attribute
   */
  @Override
  public Formula preset() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.preset()
        : this.preset;
  }

  /**
   * @return The value of the {@code require} attribute
   */
  @Override
  public Formula require() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.require()
        : this.require;
  }

  /**
   * @return The value of the {@code semantics} attribute
   */
  @Override
  public TLSF.Semantics semantics() {
    return semantics;
  }

  /**
   * @return The value of the {@code target} attribute
   */
  @Override
  public TLSF.Semantics target() {
    return target;
  }

  /**
   * @return The value of the {@code title} attribute
   */
  @Override
  public String title() {
    return title;
  }

  /**
   * @return The computed-at-construction value of the {@code toFormula} attribute
   */
  @Override
  public Formula toFormula() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.toFormula()
        : this.toFormula;
  }

  /**
   * Copy the current immutable object by setting a value for the {@link TLSF#assert_() assert_} attribute.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for assert_
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableTLSF withAssert_(Formula value) {
    if (this.assert_ == value) return this;
    Formula newValue = Preconditions.checkNotNull(value, "assert_");
    return new ImmutableTLSF(
        newValue,
        this.assume,
        this.description,
        this.guarantee,
        this.initially,
        this.inputs,
        this.mapping,
        this.outputs,
        this.preset,
        this.require,
        this.semantics,
        this.target,
        this.title);
  }

  /**
   * Copy the current immutable object by setting a value for the {@link TLSF#assume() assume} attribute.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for assume
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableTLSF withAssume(Formula value) {
    if (this.assume == value) return this;
    Formula newValue = Preconditions.checkNotNull(value, "assume");
    return new ImmutableTLSF(
        this.assert_,
        newValue,
        this.description,
        this.guarantee,
        this.initially,
        this.inputs,
        this.mapping,
        this.outputs,
        this.preset,
        this.require,
        this.semantics,
        this.target,
        this.title);
  }

  /**
   * Copy the current immutable object by setting a value for the {@link TLSF#description() description} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for description
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableTLSF withDescription(String value) {
    if (this.description.equals(value)) return this;
    String newValue = Preconditions.checkNotNull(value, "description");
    return new ImmutableTLSF(
        this.assert_,
        this.assume,
        newValue,
        this.guarantee,
        this.initially,
        this.inputs,
        this.mapping,
        this.outputs,
        this.preset,
        this.require,
        this.semantics,
        this.target,
        this.title);
  }

  /**
   * Copy the current immutable object by setting a value for the {@link TLSF#guarantee() guarantee} attribute.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for guarantee
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableTLSF withGuarantee(Formula value) {
    if (this.guarantee == value) return this;
    Formula newValue = Preconditions.checkNotNull(value, "guarantee");
    return new ImmutableTLSF(
        this.assert_,
        this.assume,
        this.description,
        newValue,
        this.initially,
        this.inputs,
        this.mapping,
        this.outputs,
        this.preset,
        this.require,
        this.semantics,
        this.target,
        this.title);
  }

  /**
   * Copy the current immutable object by setting a value for the {@link TLSF#initially() initially} attribute.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for initially
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableTLSF withInitially(Formula value) {
    if (this.initially == value) return this;
    Formula newValue = Preconditions.checkNotNull(value, "initially");
    return new ImmutableTLSF(
        this.assert_,
        this.assume,
        this.description,
        this.guarantee,
        newValue,
        this.inputs,
        this.mapping,
        this.outputs,
        this.preset,
        this.require,
        this.semantics,
        this.target,
        this.title);
  }

  /**
   * Copy the current immutable object by setting a value for the {@link TLSF#inputs() inputs} attribute.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for inputs
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableTLSF withInputs(BitSet value) {
    if (this.inputs == value) return this;
    BitSet newValue = Preconditions.checkNotNull(value, "inputs");
    return new ImmutableTLSF(
        this.assert_,
        this.assume,
        this.description,
        this.guarantee,
        this.initially,
        newValue,
        this.mapping,
        this.outputs,
        this.preset,
        this.require,
        this.semantics,
        this.target,
        this.title);
  }

  /**
   * Copy the current immutable object by replacing the {@link TLSF#mapping() mapping} map with the specified map.
   * Nulls are not permitted as keys or values.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param entries The entries to be added to the mapping map
   * @return A modified copy of {@code this} object
   */
  public final ImmutableTLSF withMapping(Map<String, ? extends Integer> entries) {
    if (this.mapping == entries) return this;
    ImmutableBiMap<String, Integer> newValue = ImmutableBiMap.copyOf(entries);
    return new ImmutableTLSF(
        this.assert_,
        this.assume,
        this.description,
        this.guarantee,
        this.initially,
        this.inputs,
        newValue,
        this.outputs,
        this.preset,
        this.require,
        this.semantics,
        this.target,
        this.title);
  }

  /**
   * Copy the current immutable object by setting a value for the {@link TLSF#outputs() outputs} attribute.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for outputs
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableTLSF withOutputs(BitSet value) {
    if (this.outputs == value) return this;
    BitSet newValue = Preconditions.checkNotNull(value, "outputs");
    return new ImmutableTLSF(
        this.assert_,
        this.assume,
        this.description,
        this.guarantee,
        this.initially,
        this.inputs,
        this.mapping,
        newValue,
        this.preset,
        this.require,
        this.semantics,
        this.target,
        this.title);
  }

  /**
   * Copy the current immutable object by setting a value for the {@link TLSF#preset() preset} attribute.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for preset
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableTLSF withPreset(Formula value) {
    if (this.preset == value) return this;
    Formula newValue = Preconditions.checkNotNull(value, "preset");
    return new ImmutableTLSF(
        this.assert_,
        this.assume,
        this.description,
        this.guarantee,
        this.initially,
        this.inputs,
        this.mapping,
        this.outputs,
        newValue,
        this.require,
        this.semantics,
        this.target,
        this.title);
  }

  /**
   * Copy the current immutable object by setting a value for the {@link TLSF#require() require} attribute.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for require
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableTLSF withRequire(Formula value) {
    if (this.require == value) return this;
    Formula newValue = Preconditions.checkNotNull(value, "require");
    return new ImmutableTLSF(
        this.assert_,
        this.assume,
        this.description,
        this.guarantee,
        this.initially,
        this.inputs,
        this.mapping,
        this.outputs,
        this.preset,
        newValue,
        this.semantics,
        this.target,
        this.title);
  }

  /**
   * Copy the current immutable object by setting a value for the {@link TLSF#semantics() semantics} attribute.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for semantics
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableTLSF withSemantics(TLSF.Semantics value) {
    if (this.semantics == value) return this;
    TLSF.Semantics newValue = Preconditions.checkNotNull(value, "semantics");
    return new ImmutableTLSF(
        this.assert_,
        this.assume,
        this.description,
        this.guarantee,
        this.initially,
        this.inputs,
        this.mapping,
        this.outputs,
        this.preset,
        this.require,
        newValue,
        this.target,
        this.title);
  }

  /**
   * Copy the current immutable object by setting a value for the {@link TLSF#target() target} attribute.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for target
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableTLSF withTarget(TLSF.Semantics value) {
    if (this.target == value) return this;
    TLSF.Semantics newValue = Preconditions.checkNotNull(value, "target");
    return new ImmutableTLSF(
        this.assert_,
        this.assume,
        this.description,
        this.guarantee,
        this.initially,
        this.inputs,
        this.mapping,
        this.outputs,
        this.preset,
        this.require,
        this.semantics,
        newValue,
        this.title);
  }

  /**
   * Copy the current immutable object by setting a value for the {@link TLSF#title() title} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for title
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableTLSF withTitle(String value) {
    if (this.title.equals(value)) return this;
    String newValue = Preconditions.checkNotNull(value, "title");
    return new ImmutableTLSF(
        this.assert_,
        this.assume,
        this.description,
        this.guarantee,
        this.initially,
        this.inputs,
        this.mapping,
        this.outputs,
        this.preset,
        this.require,
        this.semantics,
        this.target,
        newValue);
  }

  /**
   * This instance is equal to all instances of {@code ImmutableTLSF} that have equal attribute values.
   * @return {@code true} if {@code this} is equal to {@code another} instance
   */
  @Override
  public boolean equals(@Nullable Object another) {
    if (this == another) return true;
    return another instanceof ImmutableTLSF
        && equalTo((ImmutableTLSF) another);
  }

  private boolean equalTo(ImmutableTLSF another) {
    return assert_.equals(another.assert_)
        && assume.equals(another.assume)
        && description.equals(another.description)
        && guarantee.equals(another.guarantee)
        && initially.equals(another.initially)
        && inputs.equals(another.inputs)
        && mapping.equals(another.mapping)
        && outputs.equals(another.outputs)
        && preset.equals(another.preset)
        && require.equals(another.require)
        && semantics.equals(another.semantics)
        && target.equals(another.target)
        && title.equals(another.title)
        && toFormula.equals(another.toFormula);
  }

  /**
   * Computes a hash code from attributes: {@code assert_}, {@code assume}, {@code description}, {@code guarantee}, {@code initially}, {@code inputs}, {@code mapping}, {@code outputs}, {@code preset}, {@code require}, {@code semantics}, {@code target}, {@code title}, {@code toFormula}.
   * @return hashCode value
   */
  @Override
  public int hashCode() {
    int h = 31;
    h = h * 17 + assert_.hashCode();
    h = h * 17 + assume.hashCode();
    h = h * 17 + description.hashCode();
    h = h * 17 + guarantee.hashCode();
    h = h * 17 + initially.hashCode();
    h = h * 17 + inputs.hashCode();
    h = h * 17 + mapping.hashCode();
    h = h * 17 + outputs.hashCode();
    h = h * 17 + preset.hashCode();
    h = h * 17 + require.hashCode();
    h = h * 17 + semantics.hashCode();
    h = h * 17 + target.hashCode();
    h = h * 17 + title.hashCode();
    h = h * 17 + toFormula.hashCode();
    return h;
  }

  /**
   * Prints the immutable value {@code TLSF} with attribute values.
   * @return A string representation of the value
   */
  @Override
  public String toString() {
    return MoreObjects.toStringHelper("TLSF")
        .omitNullValues()
        .add("assert_", assert_)
        .add("assume", assume)
        .add("description", description)
        .add("guarantee", guarantee)
        .add("initially", initially)
        .add("inputs", inputs)
        .add("mapping", mapping)
        .add("outputs", outputs)
        .add("preset", preset)
        .add("require", require)
        .add("semantics", semantics)
        .add("target", target)
        .add("title", title)
        .add("toFormula", toFormula)
        .toString();
  }

  /**
   * Creates an immutable copy of a {@link TLSF} value.
   * Uses accessors to get values to initialize the new immutable instance.
   * If an instance is already immutable, it is returned as is.
   * @param instance The instance to copy
   * @return A copied immutable TLSF instance
   */
  public static ImmutableTLSF copyOf(TLSF instance) {
    if (instance instanceof ImmutableTLSF) {
      return (ImmutableTLSF) instance;
    }
    return ImmutableTLSF.builder()
        .from(instance)
        .build();
  }

  /**
   * Creates a builder for {@link ImmutableTLSF ImmutableTLSF}.
   * @return A new ImmutableTLSF builder
   */
  public static ImmutableTLSF.Builder builder() {
    return new ImmutableTLSF.Builder();
  }

  /**
   * Builds instances of type {@link ImmutableTLSF ImmutableTLSF}.
   * Initialize attributes and then invoke the {@link #build()} method to create an
   * immutable instance.
   * <p><em>{@code Builder} is not thread-safe and generally should not be stored in a field or collection,
   * but instead used immediately to create instances.</em>
   */
  @NotThreadSafe
  public static final class Builder {
    private static final long INIT_BIT_DESCRIPTION = 0x1L;
    private static final long INIT_BIT_INPUTS = 0x2L;
    private static final long INIT_BIT_OUTPUTS = 0x4L;
    private static final long INIT_BIT_SEMANTICS = 0x8L;
    private static final long INIT_BIT_TARGET = 0x10L;
    private static final long INIT_BIT_TITLE = 0x20L;
    private long initBits = 0x3fL;

    private @Nullable Formula assert_;
    private @Nullable Formula assume;
    private @Nullable String description;
    private @Nullable Formula guarantee;
    private @Nullable Formula initially;
    private @Nullable BitSet inputs;
    private ImmutableBiMap.Builder<String, Integer> mapping = ImmutableBiMap.builder();
    private @Nullable BitSet outputs;
    private @Nullable Formula preset;
    private @Nullable Formula require;
    private @Nullable TLSF.Semantics semantics;
    private @Nullable TLSF.Semantics target;
    private @Nullable String title;

    private Builder() {
    }

    /**
     * Fill a builder with attribute values from the provided {@code TLSF} instance.
     * Regular attribute values will be replaced with those from the given instance.
     * Absent optional values will not replace present values.
     * Collection elements and entries will be added, not replaced.
     * @param instance The instance from which to copy values
     * @return {@code this} builder for use in a chained invocation
     */
    public final Builder from(TLSF instance) {
      Preconditions.checkNotNull(instance, "instance");
      assert_(instance.assert_());
      assume(instance.assume());
      description(instance.description());
      guarantee(instance.guarantee());
      initially(instance.initially());
      inputs(instance.inputs());
      putAllMapping(instance.mapping());
      outputs(instance.outputs());
      preset(instance.preset());
      require(instance.require());
      semantics(instance.semantics());
      target(instance.target());
      title(instance.title());
      return this;
    }

    /**
     * Initializes the value for the {@link TLSF#assert_() assert_} attribute.
     * <p><em>If not set, this attribute will have a default value as returned by the initializer of {@link TLSF#assert_() assert_}.</em>
     * @param assert_ The value for assert_ 
     * @return {@code this} builder for use in a chained invocation
     */
    public final Builder assert_(Formula assert_) {
      this.assert_ = Preconditions.checkNotNull(assert_, "assert_");
      return this;
    }

    /**
     * Initializes the value for the {@link TLSF#assume() assume} attribute.
     * <p><em>If not set, this attribute will have a default value as returned by the initializer of {@link TLSF#assume() assume}.</em>
     * @param assume The value for assume 
     * @return {@code this} builder for use in a chained invocation
     */
    public final Builder assume(Formula assume) {
      this.assume = Preconditions.checkNotNull(assume, "assume");
      return this;
    }

    /**
     * Initializes the value for the {@link TLSF#description() description} attribute.
     * @param description The value for description 
     * @return {@code this} builder for use in a chained invocation
     */
    public final Builder description(String description) {
      this.description = Preconditions.checkNotNull(description, "description");
      initBits &= ~INIT_BIT_DESCRIPTION;
      return this;
    }

    /**
     * Initializes the value for the {@link TLSF#guarantee() guarantee} attribute.
     * <p><em>If not set, this attribute will have a default value as returned by the initializer of {@link TLSF#guarantee() guarantee}.</em>
     * @param guarantee The value for guarantee 
     * @return {@code this} builder for use in a chained invocation
     */
    public final Builder guarantee(Formula guarantee) {
      this.guarantee = Preconditions.checkNotNull(guarantee, "guarantee");
      return this;
    }

    /**
     * Initializes the value for the {@link TLSF#initially() initially} attribute.
     * <p><em>If not set, this attribute will have a default value as returned by the initializer of {@link TLSF#initially() initially}.</em>
     * @param initially The value for initially 
     * @return {@code this} builder for use in a chained invocation
     */
    public final Builder initially(Formula initially) {
      this.initially = Preconditions.checkNotNull(initially, "initially");
      return this;
    }

    /**
     * Initializes the value for the {@link TLSF#inputs() inputs} attribute.
     * @param inputs The value for inputs 
     * @return {@code this} builder for use in a chained invocation
     */
    public final Builder inputs(BitSet inputs) {
      this.inputs = Preconditions.checkNotNull(inputs, "inputs");
      initBits &= ~INIT_BIT_INPUTS;
      return this;
    }

    /**
     * Put one entry to the {@link TLSF#mapping() mapping} map.
     * @param key The key in the mapping map
     * @param value The associated value in the mapping map
     * @return {@code this} builder for use in a chained invocation
     */
    public final Builder putMapping(String key, int value) {
      this.mapping.put(key, value);
      return this;
    }

    /**
     * Put one entry to the {@link TLSF#mapping() mapping} map. Nulls are not permitted
     * @param entry The key and value entry
     * @return {@code this} builder for use in a chained invocation
     */
    public final Builder putMapping(Map.Entry<String, ? extends Integer> entry) {
      this.mapping.put(entry);
      return this;
    }

    /**
     * Sets or replaces all mappings from the specified map as entries for the {@link TLSF#mapping() mapping} map. Nulls are not permitted
     * @param mapping The entries that will be added to the mapping map
     * @return {@code this} builder for use in a chained invocation
     */
    public final Builder mapping(Map<String, ? extends Integer> mapping) {
      this.mapping = ImmutableBiMap.builder();
      return putAllMapping(mapping);
    }

    /**
     * Put all mappings from the specified map as entries to {@link TLSF#mapping() mapping} map. Nulls are not permitted
     * @param mapping The entries that will be added to the mapping map
     * @return {@code this} builder for use in a chained invocation
     */
    public final Builder putAllMapping(Map<String, ? extends Integer> mapping) {
      this.mapping.putAll(mapping);
      return this;
    }

    /**
     * Initializes the value for the {@link TLSF#outputs() outputs} attribute.
     * @param outputs The value for outputs 
     * @return {@code this} builder for use in a chained invocation
     */
    public final Builder outputs(BitSet outputs) {
      this.outputs = Preconditions.checkNotNull(outputs, "outputs");
      initBits &= ~INIT_BIT_OUTPUTS;
      return this;
    }

    /**
     * Initializes the value for the {@link TLSF#preset() preset} attribute.
     * <p><em>If not set, this attribute will have a default value as returned by the initializer of {@link TLSF#preset() preset}.</em>
     * @param preset The value for preset 
     * @return {@code this} builder for use in a chained invocation
     */
    public final Builder preset(Formula preset) {
      this.preset = Preconditions.checkNotNull(preset, "preset");
      return this;
    }

    /**
     * Initializes the value for the {@link TLSF#require() require} attribute.
     * <p><em>If not set, this attribute will have a default value as returned by the initializer of {@link TLSF#require() require}.</em>
     * @param require The value for require 
     * @return {@code this} builder for use in a chained invocation
     */
    public final Builder require(Formula require) {
      this.require = Preconditions.checkNotNull(require, "require");
      return this;
    }

    /**
     * Initializes the value for the {@link TLSF#semantics() semantics} attribute.
     * @param semantics The value for semantics 
     * @return {@code this} builder for use in a chained invocation
     */
    public final Builder semantics(TLSF.Semantics semantics) {
      this.semantics = Preconditions.checkNotNull(semantics, "semantics");
      initBits &= ~INIT_BIT_SEMANTICS;
      return this;
    }

    /**
     * Initializes the value for the {@link TLSF#target() target} attribute.
     * @param target The value for target 
     * @return {@code this} builder for use in a chained invocation
     */
    public final Builder target(TLSF.Semantics target) {
      this.target = Preconditions.checkNotNull(target, "target");
      initBits &= ~INIT_BIT_TARGET;
      return this;
    }

    /**
     * Initializes the value for the {@link TLSF#title() title} attribute.
     * @param title The value for title 
     * @return {@code this} builder for use in a chained invocation
     */
    public final Builder title(String title) {
      this.title = Preconditions.checkNotNull(title, "title");
      initBits &= ~INIT_BIT_TITLE;
      return this;
    }

    /**
     * Builds a new {@link ImmutableTLSF ImmutableTLSF}.
     * @return An immutable instance of TLSF
     * @throws java.lang.IllegalStateException if any required attributes are missing
     */
    public ImmutableTLSF build() {
      if (initBits != 0) {
        throw new IllegalStateException(formatRequiredAttributesMessage());
      }
      return new ImmutableTLSF(this);
    }

    private String formatRequiredAttributesMessage() {
      List<String> attributes = Lists.newArrayList();
      if ((initBits & INIT_BIT_DESCRIPTION) != 0) attributes.add("description");
      if ((initBits & INIT_BIT_INPUTS) != 0) attributes.add("inputs");
      if ((initBits & INIT_BIT_OUTPUTS) != 0) attributes.add("outputs");
      if ((initBits & INIT_BIT_SEMANTICS) != 0) attributes.add("semantics");
      if ((initBits & INIT_BIT_TARGET) != 0) attributes.add("target");
      if ((initBits & INIT_BIT_TITLE) != 0) attributes.add("title");
      return "Cannot build TLSF, some of required attributes are not set " + attributes;
    }
  }
}
